@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * main.c
- * Copyright (C) John Stebbins 2008-2015 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2017 <stebbins@stebbins>
  *
  * main.c is free software.
  *
@@ -66,7 +66,7 @@
 #include "resources.h"
 #include "presets.h"
 #include "preview.h"
-#include "ghbcompositor.h"
+#include "ui_res.h"
 
 
 /*
@@ -89,9 +89,15 @@ GtkBuilder*
 create_builder_or_die(const gchar * name)
 {
     guint res = 0;
-    GValue *gval;
     GError *error = NULL;
     const gchar *ghb_ui;
+    gsize data_size;
+
+    ghb_ui_register_resource();
+    GResource *ui_res = ghb_ui_get_resource();
+    GBytes *gbytes = g_resource_lookup_data(ui_res, "/org/handbrake/ui/ghb.ui",
+                                            0, NULL);
+    ghb_ui = g_bytes_get_data(gbytes, &data_size);
 
     const gchar *markup =
         N_("<b><big>Unable to create %s.</big></b>\n"
@@ -100,8 +106,6 @@ create_builder_or_die(const gchar * name)
         "%s");
     g_debug("create_builder_or_die()\n");
     GtkBuilder *xml = gtk_builder_new();
-    gval = ghb_resource_get("ghb-ui");
-    ghb_ui = g_value_get_string(gval);
     if (xml != NULL)
         res = gtk_builder_add_from_string(xml, ghb_ui, -1, &error);
     if (!xml || !res)
@@ -327,7 +331,7 @@ bind_audio_tree_model(signal_user_data_t *ud)
     GtkTreeSelection *selection;
 
     g_debug("bind_audio_tree_model()\n");
-    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "audio_list"));
+    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "audio_list_view"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING,
@@ -392,7 +396,7 @@ bind_subtitle_tree_model(signal_user_data_t *ud)
     GtkTreeSelection *selection;
 
     g_debug("bind_subtitle_tree_model()\n");
-    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "subtitle_list"));
+    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "subtitle_list_view"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING,
@@ -627,14 +631,20 @@ IoRedirect(signal_user_data_t *ud)
     // Set encoding to raw.
     g_io_channel_set_encoding(ud->activity_log, NULL, NULL);
     // redirect stderr to the writer end of the pipe
+
 #if defined(_WIN32)
-    // dup2 doesn't work on windows for some stupid reason
-    stderr->_file = pfd[1];
+    _dup2(pfd[1], _fileno(stderr));
 #else
-    dup2(pfd[1], /*stderr*/2);
+    dup2(pfd[1], STDERR_FILENO);
 #endif
     setvbuf(stderr, NULL, _IONBF, 0);
+
+#if defined(_WIN32)
+    channel = g_io_channel_win32_new_fd(pfd[0]);
+#else
     channel = g_io_channel_unix_new(pfd[0]);
+#endif
+
     // I was getting an this error:
     // "Invalid byte sequence in conversion input"
     // Set disable encoding on the channel.
@@ -651,12 +661,20 @@ typedef struct
 static gchar *dvd_device = NULL;
 static gchar *arg_preset = NULL;
 static gboolean ghb_debug = FALSE;
+static gchar *arg_config_dir = NULL;
+#if defined(_WIN32)
+static gboolean win32_console = FALSE;
+#endif
 
 static GOptionEntry entries[] =
 {
     { "device", 'd', 0, G_OPTION_ARG_FILENAME, &dvd_device, N_("The device or file to encode"), NULL },
     { "preset", 'p', 0, G_OPTION_ARG_STRING, &arg_preset, N_("The preset values to use for encoding"), NULL },
     { "debug",  'x', 0, G_OPTION_ARG_NONE, &ghb_debug, N_("Spam a lot"), NULL },
+    { "config", 'o', 0, G_OPTION_ARG_STRING, &arg_config_dir, N_("The path to override user config dir"), NULL },
+#if defined(_WIN32)
+    { "console",'c', 0, G_OPTION_ARG_NONE, &win32_console, N_("Open a console for debug output"), NULL },
+#endif
     { NULL }
 };
 
@@ -697,26 +715,14 @@ watch_volumes(signal_user_data_t *ud)
 
 G_MODULE_EXPORT void x264_entry_changed_cb(GtkWidget *widget, signal_user_data_t *ud);
 G_MODULE_EXPORT void video_option_changed_cb(GtkWidget *widget, signal_user_data_t *ud);
+G_MODULE_EXPORT void plot_changed_cb(GtkWidget *widget, signal_user_data_t *ud);
 G_MODULE_EXPORT void position_overlay_cb(GtkWidget *widget, signal_user_data_t *ud);
 G_MODULE_EXPORT void preview_hud_size_alloc_cb(GtkWidget *widget, signal_user_data_t *ud);
 
+// Important: any widgets named in CSS must have their widget names set
+// below before setting CSS properties on them.
 const gchar *MyCSS =
 "                                   \n\
-GtkRadioButton .button {            \n\
-    border-width: 0px;              \n\
-    padding: 0px;                   \n\
-}                                   \n\
-GtkComboBox {                       \n\
-    padding: 0px;                   \n\
-}                                   \n\
-GtkComboBox .button {               \n\
-    padding: 2px;                   \n\
-    border-width: 0px;              \n\
-}                                   \n\
-GtkEntry {                          \n\
-    padding: 4px 4px;               \n\
-}                                   \n\
-                                    \n\
 @define-color black  #000000;       \n\
 @define-color gray18 #2e2e2e;       \n\
 @define-color gray22 #383838;       \n\
@@ -726,10 +732,16 @@ GtkEntry {                          \n\
 @define-color gray46 #757575;       \n\
 @define-color white  #ffffff;       \n\
                                     \n\
-#preview_hud,                       \n\
+#preview_hud                        \n\
+{                                   \n\
+    border-radius: 20px;            \n\
+    background-color: alpha(@gray18, 0.8); \n\
+    color: @white;                  \n\
+}                                   \n\
+                                    \n\
 #live_preview_play,                 \n\
 #live_duration,                     \n\
-#preview_fullscreen                 \n\
+#preview_reset                      \n\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray18;      \n\
@@ -748,31 +760,53 @@ GtkEntry {                          \n\
 #preview_frame                      \n\
 {                                   \n\
     background: @black;             \n\
-    background-color: @gray46;      \n\
+    background-color: alpha(@gray18, 0.0); \n\
     color: @white;                  \n\
 }                                   \n\
                                     \n\
-#preview_fullscreen:prelight        \n\
+"
+#if GTK_CHECK_VERSION(3, 20, 0)
+"\
+#preview_reset:hover                \n\
+"
+#else
+"\
+#preview_reset:prelight             \n\
+"
+#endif
+"\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray32;      \n\
     color: @white;                  \n\
 }                                   \n\
                                     \n\
-#preview_fullscreen:active          \n\
+#preview_reset:active               \n\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray32;      \n\
     color: @white;                  \n\
 }                                   \n\
                                     \n\
-#preview_fullscreen:active          \n\
+#preview_reset:active               \n\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray32;      \n\
     color: @white;                  \n\
 }                                   \n\
-";
+"
+#if GTK_CHECK_VERSION(3, 16, 0)
+"                                   \n\
+                                    \n\
+#activity_view                      \n\
+{                                   \n\
+    font-family: monospace;         \n\
+    font-size: 7pt;                 \n\
+    font-weight: lighter;           \n\
+}                                   \n\
+"
+#endif
+;
 
 extern G_MODULE_EXPORT void status_icon_query_tooltip_cb(void);
 
@@ -780,10 +814,15 @@ int
 main(int argc, char *argv[])
 {
     signal_user_data_t *ud;
-    GValue *preset;
     GError *error = NULL;
     GOptionContext *context;
 
+#if defined(_WIN32)
+    // Tell gdk pixbuf where it's loader config file is.
+    _putenv_s("GDK_PIXBUF_MODULE_FILE", "ghb.exe.local/loaders.cache");
+    _putenv_s("GST_PLUGIN_PATH", "lib/gstreamer-1.0");
+#endif
+
     hb_global_init();
 
 #ifdef ENABLE_NLS
@@ -806,6 +845,26 @@ main(int argc, char *argv[])
     }
     g_option_context_free(context);
 
+#if defined(_WIN32)
+    if (win32_console)
+    {
+        // Enable console logging
+        if(AttachConsole(ATTACH_PARENT_PROCESS) || AllocConsole()){
+            close(STDOUT_FILENO);
+            freopen("CONOUT$", "w", stdout);
+            close(STDERR_FILENO);
+            freopen("CONOUT$", "w", stderr);
+        }
+    }
+    else
+    {
+        // Non-console windows apps do not have a stderr->_file
+        // assigned properly
+        stderr->_file = STDERR_FILENO;
+        stdout->_file = STDOUT_FILENO;
+    }
+#endif
+
     if (argc > 1 && dvd_device == NULL && argv[1][0] != '-')
     {
         dvd_device = argv[1];
@@ -831,7 +890,6 @@ main(int argc, char *argv[])
 #if !defined(_WIN32)
     notify_init("HandBrake");
 #endif
-    ghb_register_transforms();
     ghb_resource_init();
     ghb_load_icons();
 
@@ -840,6 +898,12 @@ main(int argc, char *argv[])
 #endif
     ghb_udev_init();
 
+    // Override user config dir
+    if (arg_config_dir != NULL)
+    {
+        ghb_override_user_config_dir(arg_config_dir);
+    }
+
     ghb_write_pid_file();
     ud = g_malloc0(sizeof(signal_user_data_t));
     ud->debug = ghb_debug;
@@ -847,10 +911,10 @@ main(int argc, char *argv[])
     g_log_set_handler("Gtk", G_LOG_LEVEL_WARNING, warn_log_handler, ud);
     //g_log_set_handler("Gtk", G_LOG_LEVEL_CRITICAL, warn_log_handler, ud);
 
-    ud->globals = ghb_settings_new();
-    ud->prefs = ghb_settings_new();
-    ud->settings_array = ghb_array_value_new(1);
-    ud->settings = ghb_settings_new();
+    ud->globals = ghb_dict_new();
+    ud->prefs = ghb_dict_new();
+    ud->settings_array = ghb_array_new();
+    ud->settings = ghb_dict_new();
     ghb_array_append(ud->settings_array, ud->settings);
 
     ud->builder = create_builder_or_die(BUILDER_NAME);
@@ -862,8 +926,8 @@ main(int argc, char *argv[])
 
     // Since GtkBuilder no longer assigns object ids to widget names
     // Assign a few that are necessary for style overrides to work
-    GtkWidget *widget;
 #if defined(_NO_UPDATE_CHECK)
+    GtkWidget *widget;
     widget = GHB_WIDGET(ud->builder, "check_updates_box");
     gtk_widget_hide(widget);
 #endif
@@ -877,52 +941,8 @@ main(int argc, char *argv[])
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_encode_progress"), "live_encode_progress");
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_duration"), "live_duration");
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_show_crop"), "preview_show_crop");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_fullscreen"), "preview_fullscreen");
-    widget = GHB_WIDGET(ud->builder, "preview_hud");
-    gtk_widget_set_name(widget, "preview_hud");
-    widget = GHB_WIDGET(ud->builder, "preview_window");
-    gtk_widget_set_name(widget, "preview_window");
-
-    // Set up the "hud" control overlay for the preview window
-    GtkWidget *preview_box, *draw, *hud, *blender;
-
-    preview_box = GHB_WIDGET(ud->builder, "preview_window_box");
-    draw = GHB_WIDGET(ud->builder, "preview_image");
-    hud = GHB_WIDGET(ud->builder, "preview_hud");
-
-#if 0 // GTK_CHECK_VERSION(3, 0, 0)
-    // This uses the new GtkOverlay widget.
-    //
-    // Unfortunately, GtkOverlay is broken in a couple of ways.
-    //
-    // First, it doesn't respect gtk_widget_shape_combine_region()
-    // on it's child overlays.  It appears to just ignore the clip
-    // mask of the child.
-    //
-    // Second, it doesn't respect window opacity.
-    //
-    // So for now, I'll just continue using my home-grown overlay
-    // widget (GhbCompositor).
-    blender = gtk_overlay_new();
-    gtk_container_add(GTK_CONTAINER(preview_box), blender);
-    gtk_container_add(GTK_CONTAINER(blender), draw);
-    gtk_widget_set_valign (hud, GTK_ALIGN_END);
-    gtk_widget_set_halign (hud, GTK_ALIGN_CENTER);
-    gtk_overlay_add_overlay(GTK_OVERLAY(blender), hud);
-
-    g_signal_connect(G_OBJECT(blender), "get-child-position",
-                    G_CALLBACK(position_overlay_cb), ud);
-
-    gtk_widget_show(blender);
-#else
-    // Set up compositing for hud
-    blender = ghb_compositor_new();
-
-    gtk_container_add(GTK_CONTAINER(preview_box), blender);
-    ghb_compositor_zlist_insert(GHB_COMPOSITOR(blender), draw, 1, 1);
-    ghb_compositor_zlist_insert(GHB_COMPOSITOR(blender), hud, 2, .85);
-    gtk_widget_show(blender);
-#endif
+    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_reset"), "preview_reset");
+    gtk_widget_set_name(GHB_WIDGET(ud->builder, "activity_view"), "activity_view");
 
     // Redirect stderr to the activity window
     ghb_preview_init(ud);
@@ -943,6 +963,10 @@ main(int argc, char *argv[])
     buffer = gtk_text_view_get_buffer(textview);
     g_signal_connect(buffer, "changed", (GCallback)video_option_changed_cb, ud);
 
+    textview = GTK_TEXT_VIEW(GHB_WIDGET(ud->builder, "MetaLongDescription"));
+    buffer = gtk_text_view_get_buffer(textview);
+    g_signal_connect(buffer, "changed", (GCallback)plot_changed_cb, ud);
+
     ghb_combo_init(ud);
 
     g_debug("ud %p\n", ud);
@@ -970,13 +994,15 @@ main(int argc, char *argv[])
     ghb_settings_init(ud->prefs, "Preferences");
     ghb_settings_init(ud->globals, "Globals");
     ghb_settings_init(ud->settings, "Initialization");
+    ghb_settings_init(ud->settings, "OneTimeInitialization");
     // Load user preferences file
     ghb_prefs_load(ud);
     // Store user preferences into ud->prefs
     ghb_prefs_to_settings(ud->prefs);
 
-    // Load all settings with default preset values
-    ghb_settings_init(ud->settings, "Presets");
+    int logLevel = ghb_dict_get_int(ud->prefs, "LoggingLevel");
+    ghb_backend_init(logLevel);
+
     // Load the presets files
     ghb_presets_load(ud);
     // Note that ghb_preset_to_settings(ud->settings) is called when
@@ -987,29 +1013,20 @@ main(int argc, char *argv[])
     // Note that ghb_settings_to_ui(ud->settings) happens when initial
     // empty title is initialized.
 
-    gint logLevel;
-    logLevel = ghb_settings_get_int(ud->prefs, "LoggingLevel");
-    ghb_backend_init(logLevel);
 
-    if (ghb_settings_get_boolean(ud->prefs, "hbfd"))
+    if (ghb_dict_get_bool(ud->prefs, "hbfd"))
     {
         ghb_hbfd(ud, TRUE);
     }
-    gchar *source = ghb_settings_get_string(ud->prefs, "default_source");
+    const gchar *source = ghb_dict_get_string(ud->prefs, "default_source");
     ghb_dvd_set_current(source, ud);
-    g_free(source);
 
     // Populate the presets tree view
-    ghb_presets_list_init(ud, NULL, 0);
+    ghb_presets_list_init(ud, NULL);
     // Get the first preset name
     if (arg_preset != NULL)
     {
-        preset = ghb_parse_preset_path(arg_preset);
-        if (preset)
-        {
-            ghb_select_preset(ud->builder, preset);
-            ghb_value_free(preset);
-        }
+        ghb_select_preset(ud->builder, arg_preset);
     }
     else
     {
@@ -1022,13 +1039,13 @@ main(int argc, char *argv[])
     if (dvd_device != NULL)
     {
         // Source overridden from command line option
-        ghb_settings_set_string(ud->globals, "scan_source", dvd_device);
+        ghb_dict_set_string(ud->globals, "scan_source", dvd_device);
         g_idle_add((GSourceFunc)ghb_idle_scan, ud);
     }
     else
     {
-        GValue *gval = ghb_settings_get_value(ud->prefs, "default_source");
-        ghb_settings_set_value(ud->globals, "scan_source", gval);
+        GhbValue *gval = ghb_dict_get_value(ud->prefs, "default_source");
+        ghb_dict_set(ud->globals, "scan_source", ghb_value_dup(gval));
     }
     // Reload and check status of the last saved queue
     g_idle_add((GSourceFunc)ghb_reload_queue, ud);
@@ -1044,14 +1061,14 @@ main(int argc, char *argv[])
 
     gint window_width, window_height;
     GdkGeometry geo = {
-        -1, -1, 1920, 768, -1, -1, 10, 10, 0, 0, GDK_GRAVITY_NORTH_WEST
+        -1, -1, 1920, 1080, -1, -1, 10, 10, 0, 0, GDK_GRAVITY_NORTH_WEST
     };
     GdkWindowHints geo_mask;
     geo_mask = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_BASE_SIZE;
     gtk_window_set_geometry_hints(GTK_WINDOW(ghb_window), ghb_window,
                                   &geo, geo_mask);
-    window_width = ghb_settings_get_int(ud->prefs, "window_width");
-    window_height = ghb_settings_get_int(ud->prefs, "window_height");
+    window_width = ghb_dict_get_int(ud->prefs, "window_width");
+    window_height = ghb_dict_get_int(ud->prefs, "window_height");
 
     /*
      * Filter objects in GtkBuilder xml
@@ -1156,11 +1173,13 @@ main(int argc, char *argv[])
     filter = GTK_FILE_FILTER(GHB_OBJECT(ud->builder, "SourceFilterAll"));
     gtk_file_chooser_set_filter(chooser, filter);
 
+#if !GTK_CHECK_VERSION(3, 16, 0)
     PangoFontDescription *font_desc;
     font_desc = pango_font_description_from_string("monospace 10");
     textview = GTK_TEXT_VIEW(GHB_WIDGET(ud->builder, "activity_view"));
     gtk_widget_override_font(GTK_WIDGET(textview), font_desc);
     pango_font_description_free(font_desc);
+#endif
 
     // Grrrr!  Gtk developers !!!hard coded!!! the width of the
     // radio buttons in GtkStackSwitcher to 100!!!
@@ -1182,6 +1201,12 @@ main(int argc, char *argv[])
     g_list_free(stack_switcher_children);
 
     gtk_window_resize(GTK_WINDOW(ghb_window), window_width, window_height);
+
+    ghb_set_custom_filter_tooltip(ud, "PictureDetelecineCustom",
+                                  "detelecine", HB_FILTER_DETELECINE);
+    ghb_set_custom_filter_tooltip(ud, "PictureCombDetectCustom",
+                                  "interlace detection", HB_FILTER_DETELECINE);
+
     gtk_widget_show(ghb_window);
 
     // Everything should be go-to-go.  Lets rock!
@@ -1189,11 +1214,11 @@ main(int argc, char *argv[])
     gtk_main();
     ghb_backend_close();
 
-    ghb_value_free(ud->queue);
-    ghb_value_free(ud->settings_array);
-    ghb_value_free(ud->prefs);
-    ghb_value_free(ud->globals);
-    ghb_value_free(ud->x264_priv);
+    ghb_value_free(&ud->queue);
+    ghb_value_free(&ud->settings_array);
+    ghb_value_free(&ud->prefs);
+    ghb_value_free(&ud->globals);
+    ghb_value_free(&ud->x264_priv);
 
     g_io_channel_unref(ud->activity_log);
     ghb_settings_close();
