@@ -1799,7 +1799,7 @@ static int decccInit( hb_work_object_t * w, hb_job_t * job )
         int height = job->title->geometry.height - job->crop[0] - job->crop[1];
         int width = job->title->geometry.width - job->crop[2] - job->crop[3];
         int safe_height = 0.8 * job->title->geometry.height;
-        hb_subtitle_add_ssa_header(w->subtitle, "Courier New",
+        hb_subtitle_add_ssa_header(w->subtitle, HB_FONT_MONO,
                                    .08 * safe_height, width, height);
     }
     // When rendering subs, we need to push rollup subtitles out
