@@ -1,6 +1,6 @@
 /* dvd.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -15,7 +15,7 @@
 #include "dvdread/ifo_print.h"
 #include "dvdread/nav_read.h"
 
-static hb_dvd_t    * hb_dvdread_init( char * path );
+static hb_dvd_t    * hb_dvdread_init( hb_handle_t * h, char * path );
 static void          hb_dvdread_close( hb_dvd_t ** _d );
 static char        * hb_dvdread_name( char * path );
 static int           hb_dvdread_title_count( hb_dvd_t * d );
@@ -106,7 +106,7 @@ static char * hb_dvdread_name( char * path )
  ***********************************************************************
  *
  **********************************************************************/
-hb_dvd_t * hb_dvdread_init( char * path )
+hb_dvd_t * hb_dvdread_init( hb_handle_t * h, char * path )
 {
     hb_dvd_t * e;
     hb_dvdread_t * d;
@@ -115,6 +115,7 @@ hb_dvd_t * hb_dvdread_init( char * path )
 
     e = calloc( sizeof( hb_dvd_t ), 1 );
     d = &(e->dvdread);
+    d->h = h;
 
     /*
      * Convert UTF-8 path to current code page on Windows
@@ -146,7 +147,7 @@ hb_dvd_t * hb_dvdread_init( char * path )
     /* Open main IFO */
     if( !( d->vmg = ifoOpen( d->reader, 0 ) ) )
     {
-        hb_error( "dvd: ifoOpen failed" );
+        hb_log( "dvd: not a dvd - trying as a stream/file instead" );
         goto fail;
     }
 
@@ -491,7 +492,7 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
         lang = lang_for_code( vts->vtsi_mat->vts_subp_attr[i].lang_code );
 
         subtitle = calloc( sizeof( hb_subtitle_t ), 1 );
-        subtitle->track = i+1;
+        subtitle->track = i;
         subtitle->id = ( ( 0x20 + position ) << 8 ) | 0xbd;
         snprintf( subtitle->lang, sizeof( subtitle->lang ), "%s",
              strlen(lang->native_name) ? lang->native_name : lang->eng_name);
@@ -623,17 +624,20 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
     switch( vts->vtsi_mat->vts_video_attr.display_aspect_ratio )
     {
         case 0:
-            title->container_aspect = 4. / 3.;
+            title->container_dar.num = 4;
+            title->container_dar.den = 3;
             break;
         case 3:
-            title->container_aspect = 16. / 9.;
+            title->container_dar.num = 16;
+            title->container_dar.den = 9;
             break;
         default:
             hb_log( "scan: unknown aspect" );
             goto fail;
     }
 
-    hb_log( "scan: aspect = %g", title->container_aspect );
+    hb_log("scan: aspect = %d:%d",
+           title->container_dar.num, title->container_dar.den);
 
     /* This title is ok so far */
     goto cleanup;
@@ -887,6 +891,7 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
                 if ( d->cell_cur > d->cell_end )
                 {
                     hb_buffer_close( &b );
+                    hb_set_work_error(d->h, HB_ERROR_READ);
                     return NULL;
                 }
                 d->in_cell = 0;
@@ -985,8 +990,9 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
             /* Wasn't a valid VOBU, try next block */
             if( ++error > 1024 )
             {
-                hb_log( "dvd: couldn't find a VOBU after 1024 blocks" );
+                hb_error( "dvd: couldn't find a VOBU after 1024 blocks" );
                 hb_buffer_close( &b );
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
 
@@ -1050,11 +1056,7 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
                 d->cur_vob_id = dsi_pack.dsi_gi.vobu_vob_idn;
                 d->cur_cell_id = dsi_pack.dsi_gi.vobu_c_idn;
 
-                if( d->cell_overlap )
-                {
-                    b->s.new_chap = hb_dvdread_is_break( d );
-                    d->cell_overlap = 0;
-                }
+                d->cell_overlap = 0;
             }
         }
 
@@ -1089,6 +1091,10 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
         }
         d->pack_len--;
     }
+    if (b != NULL)
+    {
+        b->s.new_chap = hb_dvdread_is_break( d );
+    }
 
     d->block++;
 
@@ -1143,9 +1149,7 @@ static int hb_dvdread_is_break( hb_dvdread_t * d )
     pgc_t * pgc;
     int     cell;
 
-    for( i = nr_of_ptts - 1;
-         i > 0;
-         i-- )
+    for (i = nr_of_ptts - 1; i >= 0; i--)
     {
         /* Get pgc for chapter (i+1) */
         pgc_id = d->ifo->vts_ptt_srpt->title[d->ttn-1].ptt[i].pgcn;
@@ -1153,11 +1157,11 @@ static int hb_dvdread_is_break( hb_dvdread_t * d )
         pgc    = d->ifo->vts_pgcit->pgci_srp[pgc_id-1].pgc;
         cell   = pgc->program_map[pgn-1] - 1;
 
-        if( cell <= d->cell_start )
+        if( cell < d->cell_start )
             break;
 
         // This must not match against the start cell.
-        if( pgc->cell_playback[cell].first_sector == d->block && cell != d->cell_start )
+        if (pgc->cell_playback[cell].first_sector == d->block)
         {
             return i + 1;
         }
@@ -1266,9 +1270,9 @@ char * hb_dvd_name( char * path )
     return dvd_methods->name(path);
 }
 
-hb_dvd_t * hb_dvd_init( char * path )
+hb_dvd_t * hb_dvd_init( hb_handle_t * h, char * path )
 {
-    return dvd_methods->init(path);
+    return dvd_methods->init(h, path);
 }
 
 int hb_dvd_title_count( hb_dvd_t * d )
