@@ -1,6 +1,6 @@
 /* dvdnav.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -19,7 +19,7 @@
 #define DVD_READ_CACHE 1
 
 static char        * hb_dvdnav_name( char * path );
-static hb_dvd_t    * hb_dvdnav_init( char * path );
+static hb_dvd_t    * hb_dvdnav_init( hb_handle_t * h, char * path );
 static int           hb_dvdnav_title_count( hb_dvd_t * d );
 static hb_title_t  * hb_dvdnav_title_scan( hb_dvd_t * d, int t, uint64_t min_duration );
 static int           hb_dvdnav_start( hb_dvd_t * d, hb_title_t *title, int chapter );
@@ -148,7 +148,7 @@ fail:
  ***********************************************************************
  *
  **********************************************************************/
-static hb_dvd_t * hb_dvdnav_init( char * path )
+static hb_dvd_t * hb_dvdnav_init( hb_handle_t * h, char * path )
 {
     hb_dvd_t * e;
     hb_dvdnav_t * d;
@@ -157,6 +157,7 @@ static hb_dvd_t * hb_dvdnav_init( char * path )
 
     e = calloc( sizeof( hb_dvd_t ), 1 );
     d = &(e->dvdnav);
+    d->h = h;
 
     /*
      * Convert UTF-8 path to current code page on Windows
@@ -334,6 +335,7 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     uint64_t       duration, longest;
     int            longest_pgcn, longest_pgn, longest_pgcn_end;
     const char   * name;
+    unsigned char  unused[1024];
     const char   * codec_name;
 
     hb_log( "scan: scanning title %d", t );
@@ -344,8 +346,13 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     {
         strncpy( title->name, name, sizeof( title->name ) );
     }
-    else
+
+    if (strlen(title->name) == 0)
     {
+        if( DVDUDFVolumeInfo( d->reader, title->name, sizeof( title->name ),
+                             unused, sizeof( unused ) ) )
+        {
+
         char * p_cur, * p_last = d->path;
         for( p_cur = d->path; *p_cur; p_cur++ )
         {
@@ -358,6 +365,7 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
         char *dot_term = strrchr(title->name, '.');
         if (dot_term)
             *dot_term = '\0';
+        }
     }
 
     /* VTS which our title is in */
@@ -664,7 +672,7 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
         lang = lang_for_code( ifo->vtsi_mat->vts_subp_attr[i].lang_code );
 
         subtitle = calloc( sizeof( hb_subtitle_t ), 1 );
-        subtitle->track = i+1;
+        subtitle->track = i;
         subtitle->id = ( ( 0x20 + position ) << 8 ) | 0xbd;
         snprintf( subtitle->lang, sizeof( subtitle->lang ), "%s",
              strlen(lang->native_name) ? lang->native_name : lang->eng_name);
@@ -809,17 +817,20 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     switch( ifo->vtsi_mat->vts_video_attr.display_aspect_ratio )
     {
         case 0:
-            title->container_aspect = 4. / 3.;
+            title->container_dar.num = 4;
+            title->container_dar.den = 3;
             break;
         case 3:
-            title->container_aspect = 16. / 9.;
+            title->container_dar.num = 16;
+            title->container_dar.den = 9;
             break;
         default:
             hb_log( "scan: unknown aspect" );
             goto fail;
     }
 
-    hb_log( "scan: aspect = %g", title->container_aspect );
+    hb_log("scan: aspect = %d:%d",
+           title->container_dar.num, title->container_dar.den);
 
     /* This title is ok so far */
     goto cleanup;
@@ -1312,7 +1323,7 @@ static int hb_dvdnav_main_feature( hb_dvd_t * e, hb_list_t * list_title )
             longest_duration_fallback = title->duration;
             longest_fallback = title->index;
         }
-        if ( title->duration > 90000L * 60 * 30 )
+        if ( title->duration > 90000LL * 60 * 30 )
         {
             avg_duration += title->duration;
             avg_cnt++;
@@ -1408,7 +1419,7 @@ static int hb_dvdnav_main_feature( hb_dvd_t * e, hb_list_t * list_title )
         longest = longest_title;
     }
     if ((float)longest_duration_fallback * 0.7 > longest_duration &&
-        longest_duration < 90000L * 60 * 30 )
+        longest_duration < 90000LL * 60 * 30 )
     {
         float factor = (float)avg_duration / longest_duration;
         if ( factor > 1 )
@@ -1617,6 +1628,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 hb_error( "dvd: dvdnav_sector_search failed - %s",
                         dvdnav_err_to_string(d->dvdnav) );
                 hb_buffer_close( &b );
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
             error_count++;
@@ -1624,6 +1636,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
             {
                 hb_error("dvdnav: Error, too many consecutive read errors");
                 hb_buffer_close( &b );
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
             continue;
@@ -1633,11 +1646,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
         case DVDNAV_BLOCK_OK:
             // We have received a regular block of the currently playing
             // MPEG stream.
-
-            // The muxers expect to only get chapter 2 and above
-            // They write chapter 1 when chapter 2 is detected.
-            if (chapter > 1)
-                b->s.new_chap = chapter;
+            b->s.new_chap = chapter;
             chapter = 0;
             error_count = 0;
             return b;
@@ -1716,6 +1725,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 {
                     // Transition to another title signals that we are done.
                     hb_buffer_close( &b );
+                    hb_deep_log(2, "dvdnav: vts change, found next title");
                     return NULL;
                 }
             }
@@ -1739,6 +1749,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 {
                     // Transition to another title signals that we are done.
                     hb_buffer_close( &b );
+                    hb_deep_log(2, "dvdnav: cell change, found next title");
                     return NULL;
                 }
                 c = FindChapterIndex(d->list_chapter, pgcn, pgn);
@@ -1749,6 +1760,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                         // Some titles end with a 'link' back to the beginning so
                         // a transition to an earlier chapter means we're done.
                         hb_buffer_close( &b );
+                        hb_deep_log(2, "dvdnav: cell change, previous chapter");
                         return NULL;
                     }
                     chapter = d->chapter = c;
@@ -1756,6 +1768,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 else if ( cell_event->cellN <= d->cell )
                 {
                     hb_buffer_close( &b );
+                    hb_deep_log(2, "dvdnav: cell change, previous cell");
                     return NULL;
                 }
                 d->cell = cell_event->cellN;
@@ -1774,12 +1787,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
 
             // mpegdemux expects to get these.  I don't think it does
             // anything useful with them however.
-
-            // The muxers expect to only get chapter 2 and above
-            // They write chapter 1 when chapter 2 is detected.
-            if (chapter > 1)
-                b->s.new_chap = chapter;
-            chapter = 0;
+            b->s.new_chap = chapter;
             return b;
 
             break;
@@ -1798,6 +1806,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
             */
             d->stopped = 1;
             hb_buffer_close( &b );
+            hb_deep_log(2, "dvdnav: stop");
             return NULL;
 
         default:
