@@ -520,3 +520,12 @@ void                 hb_chapter_queue_close(hb_chapter_queue_t **_q);
 void                 hb_chapter_enqueue(hb_chapter_queue_t *q, hb_buffer_t *b);
 void                 hb_chapter_dequeue(hb_chapter_queue_t *q, hb_buffer_t *b);
 
+/* Font names used for rendering subtitles */
+#if defined(SYS_MINGW)
+#define HB_FONT_MONO "Lucida Console"
+#define HB_FONT_SANS "sans-serif"
+#else
+#define HB_FONT_MONO "monospace"
+#define HB_FONT_SANS "sans-serif"
+#endif
+
