@@ -1,7 +1,7 @@
 /* nlmeans.c
 
    Copyright (c) 2013 Dirk Farin
-   Copyright (c) 2003-2014 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -44,7 +44,7 @@
  *  1026: Mean 5x5 plus edge boost
  *  1281: Mean 3x3 reduced by 25% plus edge boost
  *        etc...
- *  2049: Mean 3x3 passthru (NL-means off, prefilter is the output)
+ *  2049: Mean 3x3 passthru (NLMeans off, prefilter is the output)
  *        etc...
  *  3329: Mean 3x3 reduced by 25% plus edge boost, passthru
  *        etc...
@@ -53,9 +53,10 @@
 #include "hb.h"
 #include "hbffmpeg.h"
 #include "taskset.h"
+#include "nlmeans.h"
 
-#define NLMEANS_STRENGTH_LUMA_DEFAULT      8
-#define NLMEANS_STRENGTH_CHROMA_DEFAULT    8
+#define NLMEANS_STRENGTH_LUMA_DEFAULT      6
+#define NLMEANS_STRENGTH_CHROMA_DEFAULT    6
 #define NLMEANS_ORIGIN_TUNE_LUMA_DEFAULT   1
 #define NLMEANS_ORIGIN_TUNE_CHROMA_DEFAULT 1
 #define NLMEANS_PATCH_SIZE_LUMA_DEFAULT    7
@@ -130,6 +131,12 @@ struct hb_filter_private_s
     int    nframes[3];     // temporal search depth in frames
     int    prefilter[3];   // prefilter mode, can improve weight analysis
 
+    float  exptable[3][NLMEANS_EXPSIZE];
+    float  weight_fact_table[3];
+    int    diff_max[3];
+
+    NLMeansFunctions functions;
+
     Frame      *frame;
     int         next_frame;
     int         max_frames;
@@ -147,23 +154,35 @@ static void nlmeans_close(hb_filter_object_t *filter);
 
 static void nlmeans_filter_thread(void *thread_args_v);
 
+static const char nlmeans_template[] =
+    "y-strength=^"HB_FLOAT_REG"$:y-origin-tune=^"HB_FLOAT_REG"$:"
+    "y-patch-size=^"HB_INT_REG"$:y-range=^"HB_INT_REG"$:"
+    "y-frame-count=^"HB_INT_REG"$:y-prefilter=^"HB_INT_REG"$:"
+    "cb-strength=^"HB_FLOAT_REG"$:cb-origin-tune=^"HB_FLOAT_REG"$:"
+    "cb-patch-size=^"HB_INT_REG"$:cb-range=^"HB_INT_REG"$:"
+    "cb-frame-count=^"HB_INT_REG"$:cb-prefilter=^"HB_INT_REG"$:"
+    "cr-strength=^"HB_FLOAT_REG"$:cr-origin-tune=^"HB_FLOAT_REG"$:"
+    "cr-patch-size=^"HB_INT_REG"$:cr-range=^"HB_INT_REG"$:"
+    "cr-frame-count=^"HB_INT_REG"$:cr-prefilter=^"HB_INT_REG"$";
+
 hb_filter_object_t hb_filter_nlmeans =
 {
-    .id            = HB_FILTER_NLMEANS,
-    .enforce_order = 1,
-    .name          = "Denoise (nlmeans)",
-    .settings      = NULL,
-    .init          = nlmeans_init,
-    .work          = nlmeans_work,
-    .close         = nlmeans_close,
+    .id                = HB_FILTER_NLMEANS,
+    .enforce_order     = 1,
+    .name              = "Denoise (nlmeans)",
+    .settings          = NULL,
+    .init              = nlmeans_init,
+    .work              = nlmeans_work,
+    .close             = nlmeans_close,
+    .settings_template = nlmeans_template,
 };
 
 static void nlmeans_border(uint8_t *src,
-                           int w,
-                           int h,
-                           int border)
+                           const int w,
+                           const int h,
+                           const int border)
 {
-    int bw = w + 2 * border;
+    const int bw = w + 2 * border;
     uint8_t *image = src + border + bw * border;
 
     // Create faux borders using edge pixels
@@ -183,17 +202,20 @@ static void nlmeans_border(uint8_t *src,
 
 }
 
-static void nlmeans_deborder(BorderedPlane *src,
+static void nlmeans_deborder(const BorderedPlane *src,
                              uint8_t *dst,
-                             int w,
-                             int s,
-                             int h)
+                             const int w,
+                             const int s,
+                             const int h)
 {
-    int bw = src->w + 2 * src->border;
+    const int bw = src->w + 2 * src->border;
     uint8_t *image = src->mem + src->border + bw * src->border;
+
     int width = w;
     if (src->w < width)
+    {
         width = src->w;
+    }
 
     // Copy main image
     for (int y = 0; y < h; y++)
@@ -203,15 +225,15 @@ static void nlmeans_deborder(BorderedPlane *src,
 
 }
 
-static void nlmeans_alloc(uint8_t *src,
-                          int src_w,
-                          int src_s,
-                          int src_h,
+static void nlmeans_alloc(const uint8_t *src,
+                          const int src_w,
+                          const int src_s,
+                          const int src_h,
                           BorderedPlane *dst,
-                          int border)
+                          const int border)
 {
-    int bw = src_w + 2 * border;
-    int bh = src_h + 2 * border;
+    const int bw = src_w + 2 * border;
+    const int bh = src_h + 2 * border;
 
     uint8_t *mem   = malloc(bw * bh * sizeof(uint8_t));
     uint8_t *image = mem + border + bw * border;
@@ -234,20 +256,20 @@ static void nlmeans_alloc(uint8_t *src,
 
 }
 
-static void nlmeans_filter_mean(uint8_t *src,
-                                uint8_t *dst,
-                                int w,
-                                int h,
-                                int border,
-                                int size)
+static void nlmeans_filter_mean(const uint8_t *src,
+                                      uint8_t *dst,
+                                const int w,
+                                const int h,
+                                const int border,
+                                const int size)
 {
 
     // Mean filter
-    int bw = w + 2 * border;
-    int offset_min = -((size - 1) /2);
-    int offset_max =   (size + 1) /2;
+    const int bw = w + 2 * border;
+    const int offset_min = -((size - 1) /2);
+    const int offset_max =   (size + 1) /2;
+    const double pixel_weight = 1.0 / (size * size);
     uint16_t pixel_sum;
-    double pixel_weight = 1.0 / (size * size);
     for (int y = 0; y < h; y++)
     {
         for (int x = 0; x < w; x++)
@@ -266,7 +288,8 @@ static void nlmeans_filter_mean(uint8_t *src,
 
 }
 
-static uint8_t nlmeans_filter_median_opt(uint8_t *pixels, int size)
+static uint8_t nlmeans_filter_median_opt(uint8_t  *pixels,
+                                         const int size)
 {
 
     // Optimized sorting networks
@@ -330,17 +353,17 @@ static uint8_t nlmeans_filter_median_opt(uint8_t *pixels, int size)
 
 }
 
-static void nlmeans_filter_median(uint8_t *src,
-                                  uint8_t *dst,
-                                  int w,
-                                  int h,
-                                  int border,
-                                  int size)
+static void nlmeans_filter_median(const uint8_t *src,
+                                        uint8_t *dst,
+                                  const int w,
+                                  const int h,
+                                  const int border,
+                                  const int size)
 {
     // Median filter
-    int bw = w + 2 * border;
-    int offset_min = -((size - 1) /2);
-    int offset_max =   (size + 1) /2;
+    const int bw = w + 2 * border;
+    const int offset_min = -((size - 1) /2);
+    const int offset_max =   (size + 1) /2;
     int index;
     uint8_t pixels[size * size];
     for (int y = 0; y < h; y++)
@@ -362,25 +385,25 @@ static void nlmeans_filter_median(uint8_t *src,
 
 }
 
-static void nlmeans_filter_edgeboost(uint8_t *src,
-                                     uint8_t *dst,
-                                     int w,
-                                     int h,
-                                     int border)
+static void nlmeans_filter_edgeboost(const uint8_t *src,
+                                           uint8_t *dst,
+                                     const int w,
+                                     const int h,
+                                     const int border)
 {
-    int bw = w + 2 * border;
-    int bh = h + 2 * border;
+    const int bw = w + 2 * border;
+    const int bh = h + 2 * border;
 
     // Custom kernel
-    int kernel_size = 3;
-    int kernel[3][3] = {{-31, 0, 31},
-                        {-44, 0, 44},
-                        {-31, 0, 31}};
-    double kernel_coef = 1.0 / 126.42;
+    const int kernel_size = 3;
+    const int kernel[3][3] = {{-31, 0, 31},
+                              {-44, 0, 44},
+                              {-31, 0, 31}};
+    const double kernel_coef = 1.0 / 126.42;
 
     // Detect edges
-    int offset_min = -((kernel_size - 1) /2);
-    int offset_max =   (kernel_size + 1) /2;
+    const int offset_min = -((kernel_size - 1) /2);
+    const int offset_max =   (kernel_size + 1) /2;
     uint16_t pixel1;
     uint16_t pixel2;
     uint8_t *mask_mem = calloc(bw * bh, sizeof(uint8_t));
@@ -466,7 +489,7 @@ static void nlmeans_filter_edgeboost(uint8_t *src,
 }
 
 static void nlmeans_prefilter(BorderedPlane *src,
-                              int filter_type)
+                              const int filter_type)
 {
     hb_lock(src->mutex);
     if (src->prefiltered)
@@ -482,13 +505,13 @@ static void nlmeans_prefilter(BorderedPlane *src,
     {
 
         // Source image
-        uint8_t *mem   = src->mem;
-        uint8_t *image = src->image;
-        int border     = src->border;
-        int w          = src->w;
-        int h          = src->h;
-        int bw         = w + 2 * border;
-        int bh         = h + 2 * border;
+        const uint8_t *mem   = src->mem;
+        const uint8_t *image = src->image;
+        const int border     = src->border;
+        const int w          = src->w;
+        const int h          = src->h;
+        const int bw         = w + 2 * border;
+        const int bh         = h + 2 * border;
 
         // Duplicate plane
         uint8_t *mem_pre = malloc(bw * bh * sizeof(uint8_t));
@@ -568,48 +591,82 @@ static void nlmeans_prefilter(BorderedPlane *src,
     hb_unlock(src->mutex);
 }
 
-static void nlmeans_plane(Frame *frame,
+static void build_integral_scalar(uint32_t *integral,
+                                  int       integral_stride,
+                            const uint8_t  *src,
+                            const uint8_t  *src_pre,
+                            const uint8_t  *compare,
+                            const uint8_t  *compare_pre,
+                                  int       w,
+                                  int       border,
+                                  int       dst_w,
+                                  int       dst_h,
+                                  int       dx,
+                                  int       dy)
+{
+    const int bw = w + 2 * border;
+    for (int y = 0; y < dst_h; y++)
+    {
+        const uint8_t *p1 = src_pre + y*bw;
+        const uint8_t *p2 = compare_pre + (y+dy)*bw + dx;
+        uint32_t *out = integral + (y*integral_stride);
+
+        for (int x = 0; x < dst_w; x++)
+        {
+            int diff = *p1 - *p2;
+            *out = *(out-1) + diff * diff;
+            out++;
+            p1++;
+            p2++;
+        }
+
+        if (y > 0)
+        {
+            out = integral + y*integral_stride;
+
+            for (int x = 0; x < dst_w; x++)
+            {
+                *out += *(out - integral_stride);
+                out++;
+            }
+        }
+    }
+}
+
+static void nlmeans_plane(NLMeansFunctions *functions,
+                          Frame *frame,
                           int prefilter,
                           int plane,
                           int nframes,
                           uint8_t *dst,
-                          int w,
-                          int s,
-                          int h,
+                          int dst_w,
+                          int dst_s,
+                          int dst_h,
                           double h_param,
                           double origin_tune,
                           int n,
-                          int r)
+                          int r,
+                    const float *exptable,
+                    const float  weight_fact_table,
+                    const int    diff_max)
 {
-    int n_half = (n-1) /2;
-    int r_half = (r-1) /2;
+    const int n_half = (n-1) /2;
+    const int r_half = (r-1) /2;
 
     // Source image
-    uint8_t *src     = frame[0].plane[plane].image;
-    uint8_t *src_pre = frame[0].plane[plane].image_pre;
-    int border       = frame[0].plane[plane].border;
-    int src_w        = frame[0].plane[plane].w + 2 * border;
+    const uint8_t *src     = frame[0].plane[plane].image;
+    const uint8_t *src_pre = frame[0].plane[plane].image_pre;
+    const int w      = frame[0].plane[plane].w;
+    const int border = frame[0].plane[plane].border;
+    const int bw     = w + 2 * border;
 
     // Allocate temporary pixel sums
-    struct PixelSum *tmp_data = calloc(w * h, sizeof(struct PixelSum));
+    struct PixelSum *tmp_data = calloc(dst_w * dst_h, sizeof(struct PixelSum));
 
     // Allocate integral image
-    int integral_stride = w + 2 * 16;
-    uint32_t *integral_mem = malloc(integral_stride * (h+1) * sizeof(uint32_t));
-    uint32_t *integral     = integral_mem + integral_stride + 16;
-
-    // Precompute exponential table
-    float exptable[NLMEANS_EXPSIZE];
-    const float weight_factor       = 1.0/n/n / (h_param * h_param);
-    const float min_weight_in_table = 0.0005;
-    const float stretch             = NLMEANS_EXPSIZE / (-log(min_weight_in_table));
-    const float weight_fact_table   = weight_factor * stretch;
-    const int   diff_max            = NLMEANS_EXPSIZE / weight_fact_table;
-    for (int i = 0; i < NLMEANS_EXPSIZE; i++)
-    {
-        exptable[i] = exp(-i/stretch);
-    }
-    exptable[NLMEANS_EXPSIZE-1] = 0;
+    const int integral_stride    = ((dst_w + 15) / 16 * 16) + 2 * 16;
+    uint32_t* const integral_mem = calloc(integral_stride * (dst_h+1), sizeof(uint32_t));
+    uint32_t* const integral     = integral_mem + integral_stride + 16;
 
     // Iterate through available frames
     for (int f = 0; f < nframes; f++)
@@ -617,10 +674,8 @@ static void nlmeans_plane(Frame *frame,
         nlmeans_prefilter(&frame[f].plane[plane], prefilter);
 
         // Compare image
-        uint8_t *compare     = frame[f].plane[plane].image;
-        uint8_t *compare_pre = frame[f].plane[plane].image_pre;
-        int border           = frame[f].plane[plane].border;
-        int compare_w        = frame[f].plane[plane].w + 2 * border;
+        const uint8_t *compare     = frame[f].plane[plane].image;
+        const uint8_t *compare_pre = frame[f].plane[plane].image_pre;
 
         // Iterate through all displacements
         for (int dy = -r_half; dy <= r_half; dy++)
@@ -632,71 +687,56 @@ static void nlmeans_plane(Frame *frame,
                 if (dx == 0 && dy == 0 && f == 0)
                 {
                     // TODO: Parallelize this
-                    for (int y = n_half; y < h-n + n_half; y++)
+                    for (int y = n_half; y < dst_h-n + n_half; y++)
                     {
-                        for (int x = n_half; x < w-n + n_half; x++)
+                        for (int x = n_half; x < dst_w-n + n_half; x++)
                         {
-                            tmp_data[y*w + x].weight_sum += origin_tune;
-                            tmp_data[y*w + x].pixel_sum  += origin_tune * src[y*src_w + x];
+                            tmp_data[y*dst_w + x].weight_sum += origin_tune;
+                            tmp_data[y*dst_w + x].pixel_sum  += origin_tune * src[y*bw + x];
                         }
                     }
                     continue;
                 }
 
                 // Build integral
-                memset(integral-1 - integral_stride, 0, (w+1) * sizeof(uint32_t));
-                for (int y = 0; y < h; y++)
-                {
-                    const uint8_t *p1 = src_pre + y*src_w;
-                    const uint8_t *p2 = compare_pre + (y+dy)*compare_w + dx;
-                    uint32_t *out = integral + (y*integral_stride) - 1;
-
-                    *out++ = 0;
-
-                    for (int x = 0; x < w; x++)
-                    {
-                        int diff = *p1++ - *p2++;
-                        *out = *(out-1) + diff * diff;
-                        out++;
-                    }
-
-                    if (y > 0)
-                    {
-                        out = integral + y*integral_stride;
-
-                        for (int x = 0; x < w; x++)
-                        {
-                            *out += *(out - integral_stride);
-                            out++;
-                        }
-                    }
-                }
+                functions->build_integral(integral,
+                                          integral_stride,
+                                          src,
+                                          src_pre,
+                                          compare,
+                                          compare_pre,
+                                          w,
+                                          border,
+                                          dst_w,
+                                          dst_h,
+                                          dx,
+                                          dy);
 
                 // Average displacement
                 // TODO: Parallelize this
-                for (int y = 0; y <= h-n; y++)
+                for (int y = 0; y <= dst_h-n; y++)
                 {
                     const uint32_t *integral_ptr1 = integral + (y  -1)*integral_stride - 1;
                     const uint32_t *integral_ptr2 = integral + (y+n-1)*integral_stride - 1;
 
-                    for (int x = 0; x <= w-n; x++)
+                    for (int x = 0; x <= dst_w-n; x++)
                     {
-                        int xc = x + n_half;
-                        int yc = y + n_half;
+                        const int xc = x + n_half;
+                        const int yc = y + n_half;
 
                         // Difference between patches
-                        int diff = (uint32_t)(integral_ptr2[n] - integral_ptr2[0] - integral_ptr1[n] + integral_ptr1[0]);
+                        const int diff = (uint32_t)(integral_ptr2[n] - integral_ptr2[0] - integral_ptr1[n] + integral_ptr1[0]);
 
                         // Sum pixel with weight
                         if (diff < diff_max)
                         {
-                            int diffidx = diff * weight_fact_table;
+                            const int diffidx = diff * weight_fact_table;
 
                             //float weight = exp(-diff*weightFact);
-                            float weight = exptable[diffidx];
+                            const float weight = exptable[diffidx];
 
-                            tmp_data[yc*w + xc].weight_sum += weight;
-                            tmp_data[yc*w + xc].pixel_sum  += weight * compare[(yc+dy)*compare_w + xc + dx];
+                            tmp_data[yc*dst_w + xc].weight_sum += weight;
+                            tmp_data[yc*dst_w + xc].pixel_sum  += weight * compare[(yc+dy)*bw + xc + dx];
                         }
 
                         integral_ptr1++;
@@ -708,28 +748,28 @@ static void nlmeans_plane(Frame *frame,
     }
 
     // Copy edges
-    for (int y = 0; y < h; y++)
+    for (int y = 0; y < dst_h; y++)
     {
         for (int x = 0; x < n_half; x++)
         {
-            *(dst + y * s + x)           = *(src + y * src_w - x - 1);
-            *(dst + y * s - x + (w - 1)) = *(src + y * src_w + x + w);
+            *(dst + y * dst_s + x)               = *(src + y * bw - x - 1);
+            *(dst + y * dst_s - x + (dst_w - 1)) = *(src + y * bw + x + dst_w);
         }
     }
     for (int y = 0; y < n_half; y++)
     {
-        memcpy(dst +       y*s, src - (y+1)*src_w, w);
-        memcpy(dst + (h-y-1)*s, src + (y+h)*src_w, w);
+        memcpy(dst +           y*dst_s, src -     (y+1)*bw, dst_w);
+        memcpy(dst + (dst_h-y-1)*dst_s, src + (y+dst_h)*bw, dst_w);
     }
 
     // Copy main image
     uint8_t result;
-    for (int y = n_half; y < h-n_half; y++)
+    for (int y = n_half; y < dst_h-n_half; y++)
     {
-        for (int x = n_half; x < w-n_half; x++)
+        for (int x = n_half; x < dst_w-n_half; x++)
         {
-            result = (uint8_t)(tmp_data[y*w + x].pixel_sum / tmp_data[y*w + x].weight_sum);
-            *(dst + y*s + x) = result ? result : *(src + y*src_w + x);
+            result = (uint8_t)(tmp_data[y*dst_w + x].pixel_sum / tmp_data[y*dst_w + x].weight_sum);
+            *(dst + y*dst_s + x) = result ? result : *(src + y*bw + x);
         }
     }
 
@@ -743,6 +783,12 @@ static int nlmeans_init(hb_filter_object_t *filter,
 {
     filter->private_data = calloc(sizeof(struct hb_filter_private_s), 1);
     hb_filter_private_t *pv = filter->private_data;
+    NLMeansFunctions *functions = &pv->functions;
+
+    functions->build_integral = build_integral_scalar;
+#if defined(ARCH_X86)
+    nlmeans_init_x86(functions);
+#endif
 
     // Mark parameters unset
     for (int c = 0; c < 3; c++)
@@ -758,10 +804,27 @@ static int nlmeans_init(hb_filter_object_t *filter,
     // Read user parameters
     if (filter->settings != NULL)
     {
-        sscanf(filter->settings, "%lf:%lf:%d:%d:%d:%d:%lf:%lf:%d:%d:%d:%d:%lf:%lf:%d:%d:%d:%d",
-               &pv->strength[0], &pv->origin_tune[0], &pv->patch_size[0], &pv->range[0], &pv->nframes[0], &pv->prefilter[0],
-               &pv->strength[1], &pv->origin_tune[1], &pv->patch_size[1], &pv->range[1], &pv->nframes[1], &pv->prefilter[1],
-               &pv->strength[2], &pv->origin_tune[2], &pv->patch_size[2], &pv->range[2], &pv->nframes[2], &pv->prefilter[2]);
+        hb_dict_t * dict = filter->settings;
+        hb_dict_extract_double(&pv->strength[0],    dict, "y-strength");
+        hb_dict_extract_double(&pv->origin_tune[0], dict, "y-origin-tune");
+        hb_dict_extract_int(&pv->patch_size[0],     dict, "y-patch-size");
+        hb_dict_extract_int(&pv->range[0],          dict, "y-range");
+        hb_dict_extract_int(&pv->nframes[0],        dict, "y-frame-count");
+        hb_dict_extract_int(&pv->prefilter[0],      dict, "y-prefilter");
+
+        hb_dict_extract_double(&pv->strength[1],    dict, "cb-strength");
+        hb_dict_extract_double(&pv->origin_tune[1], dict, "cb-origin-tune");
+        hb_dict_extract_int(&pv->patch_size[1],     dict, "cb-patch-size");
+        hb_dict_extract_int(&pv->range[1],          dict, "cb-range");
+        hb_dict_extract_int(&pv->nframes[1],        dict, "cb-frame-count");
+        hb_dict_extract_int(&pv->prefilter[1],      dict, "cb-prefilter");
+
+        hb_dict_extract_double(&pv->strength[2],    dict, "cr-strength");
+        hb_dict_extract_double(&pv->origin_tune[2], dict, "cr-origin-tune");
+        hb_dict_extract_int(&pv->patch_size[2],     dict, "cr-patch-size");
+        hb_dict_extract_int(&pv->range[2],          dict, "cr-range");
+        hb_dict_extract_int(&pv->nframes[2],        dict, "cr-frame-count");
+        hb_dict_extract_int(&pv->prefilter[2],      dict, "cr-prefilter");
     }
 
     // Cascade values
@@ -799,6 +862,21 @@ static int nlmeans_init(hb_filter_object_t *filter,
         if (pv->prefilter[c] < 0)       { pv->prefilter[c] = 0; }
 
         if (pv->max_frames < pv->nframes[c]) pv->max_frames = pv->nframes[c];
+
+        // Precompute exponential table
+        float *exptable = &pv->exptable[c][0];
+        float *weight_fact_table = &pv->weight_fact_table[c];
+        int   *diff_max = &pv->diff_max[c];
+        const float weight_factor        = 1.0/pv->patch_size[c]/pv->patch_size[c] / (pv->strength[c] * pv->strength[c]);
+        const float min_weight_in_table  = 0.0005;
+        const float stretch              = NLMEANS_EXPSIZE / (-log(min_weight_in_table));
+        *(weight_fact_table)             = weight_factor * stretch;
+        *(diff_max)                      = NLMEANS_EXPSIZE / *(weight_fact_table);
+        for (int i = 0; i < NLMEANS_EXPSIZE; i++)
+        {
+            exptable[i] = exp(-i/stretch);
+        }
+        exptable[NLMEANS_EXPSIZE-1] = 0;
     }
 
     pv->thread_count = hb_get_cpu_count();
@@ -815,7 +893,7 @@ static int nlmeans_init(hb_filter_object_t *filter,
     if (taskset_init(&pv->taskset, pv->thread_count,
                      sizeof(nlmeans_thread_arg_t)) == 0)
     {
-        hb_error("nlmeans could not initialize taskset");
+        hb_error("NLMeans could not initialize taskset");
         goto fail;
     }
 
@@ -824,7 +902,7 @@ static int nlmeans_init(hb_filter_object_t *filter,
         pv->thread_data[ii] = taskset_thread_args(&pv->taskset, ii);
         if (pv->thread_data[ii] == NULL)
         {
-            hb_error("nlmeans could not create thread args");
+            hb_error("NLMeans could not create thread args");
             goto fail;
         }
         pv->thread_data[ii]->pv = pv;
@@ -832,7 +910,7 @@ static int nlmeans_init(hb_filter_object_t *filter,
         if (taskset_thread_spawn(&pv->taskset, ii, "nlmeans_filter",
                                  nlmeans_filter_thread, HB_NORMAL_PRIORITY) == 0)
         {
-            hb_error("nlmeans could not spawn thread");
+            hb_error("NLMeans could not spawn thread");
             goto fail;
         }
     }
@@ -894,7 +972,7 @@ static void nlmeans_filter_thread(void *thread_args_v)
     hb_filter_private_t *pv = thread_data->pv;
     int segment = thread_data->segment;
 
-    hb_log("NLMeans Denoise thread started for segment %d", segment);
+    hb_log("NLMeans thread started for segment %d", segment);
 
     while (1)
     {
@@ -910,6 +988,8 @@ static void nlmeans_filter_thread(void *thread_args_v)
         hb_buffer_t *buf;
         buf = hb_frame_buffer_init(frame->fmt, frame->width, frame->height);
 
+        NLMeansFunctions *functions = &pv->functions;
+
         for (int c = 0; c < 3; c++)
         {
             if (pv->strength[c] == 0)
@@ -929,7 +1009,8 @@ static void nlmeans_filter_thread(void *thread_args_v)
             }
 
             // Process current plane
-            nlmeans_plane(frame,
+            nlmeans_plane(functions,
+                          frame,
                           pv->prefilter[c],
                           c,
                           pv->nframes[c],
@@ -940,7 +1021,10 @@ static void nlmeans_filter_thread(void *thread_args_v)
                           pv->strength[c],
                           pv->origin_tune[c],
                           pv->patch_size[c],
-                          pv->range[c]);
+                          pv->range[c],
+                          pv->exptable[c],
+                          pv->weight_fact_table[c],
+                          pv->diff_max[c]);
         }
         buf->s = pv->frame[segment].s;
         thread_data->out = buf;
@@ -956,7 +1040,7 @@ static void nlmeans_add_frame(hb_filter_private_t *pv, hb_buffer_t *buf)
     for (int c = 0; c < 3; c++)
     {
         // Extend copy of plane with extra border and place in buffer
-        int border = ((pv->range[c] + 2) / 2 + 15) / 16 * 16;
+        const int border = ((pv->range[c] + 2) / 2 + 15) / 16 * 16;
         nlmeans_alloc(buf->plane[c].data,
                       buf->plane[c].width,
                       buf->plane[c].stride,
@@ -974,7 +1058,9 @@ static void nlmeans_add_frame(hb_filter_private_t *pv, hb_buffer_t *buf)
 static hb_buffer_t * nlmeans_filter(hb_filter_private_t *pv)
 {
     if (pv->next_frame < pv->max_frames + pv->thread_count)
+    {
         return NULL;
+    }
 
     taskset_cycle(&pv->taskset);
 
@@ -1013,32 +1099,28 @@ static hb_buffer_t * nlmeans_filter(hb_filter_private_t *pv)
     pv->next_frame -= pv->thread_count;
 
     // Collect results from taskset
-    hb_buffer_t *last = NULL, *out = NULL;
+    hb_buffer_list_t list;
+    hb_buffer_list_clear(&list);
     for (int t = 0; t < pv->thread_count; t++)
     {
-        if (out == NULL)
-        {
-            out = last = pv->thread_data[t]->out;
-        }
-        else
-        {
-            last->next = pv->thread_data[t]->out;
-            last = pv->thread_data[t]->out;
-        }
+        hb_buffer_list_append(&list, pv->thread_data[t]->out);
     }
-    return out;
+    return hb_buffer_list_clear(&list);
 }
 
 static hb_buffer_t * nlmeans_filter_flush(hb_filter_private_t *pv)
 {
-    hb_buffer_t *out = NULL, *last = NULL;
+    hb_buffer_list_t list;
 
+    hb_buffer_list_clear(&list);
     for (int f = 0; f < pv->next_frame; f++)
     {
         Frame *frame = &pv->frame[f];
         hb_buffer_t *buf;
         buf = hb_frame_buffer_init(frame->fmt, frame->width, frame->height);
 
+        NLMeansFunctions *functions = &pv->functions;
+
         for (int c = 0; c < 3; c++)
         {
             if (pv->strength[c] == 0)
@@ -1059,9 +1141,12 @@ static hb_buffer_t * nlmeans_filter_flush(hb_filter_private_t *pv)
 
             int nframes = pv->next_frame - f;
             if (pv->nframes[c] < nframes)
+            {
                 nframes = pv->nframes[c];
+            }
             // Process current plane
-            nlmeans_plane(frame,
+            nlmeans_plane(functions,
+                          frame,
                           pv->prefilter[c],
                           c,
                           nframes,
@@ -1072,20 +1157,15 @@ static hb_buffer_t * nlmeans_filter_flush(hb_filter_private_t *pv)
                           pv->strength[c],
                           pv->origin_tune[c],
                           pv->patch_size[c],
-                          pv->range[c]);
+                          pv->range[c],
+                          pv->exptable[c],
+                          pv->weight_fact_table[c],
+                          pv->diff_max[c]);
         }
         buf->s = frame->s;
-        if (out == NULL)
-        {
-            out = last = buf;
-        }
-        else
-        {
-            last->next = buf;
-            last = buf;
-        }
+        hb_buffer_list_append(&list, buf);
     }
-    return out;
+    return hb_buffer_list_clear(&list);
 }
 
 static int nlmeans_work(hb_filter_object_t *filter,
@@ -1095,23 +1175,19 @@ static int nlmeans_work(hb_filter_object_t *filter,
     hb_filter_private_t *pv = filter->private_data;
     hb_buffer_t *in = *buf_in;
 
-    if (in->size <= 0)
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
-        hb_buffer_t *last;
+        hb_buffer_list_t list;
+        hb_buffer_t *buf;
+
         // Flush buffered frames
-        last = *buf_out = nlmeans_filter_flush(pv);
+        buf = nlmeans_filter_flush(pv);
+        hb_buffer_list_set(&list, buf);
+
+        // And terminate the buffer list with a EOF buffer
+        hb_buffer_list_append(&list, in);
+        *buf_out = hb_buffer_list_clear(&list);
 
-        // And terminate the buffer list with a null buffer
-        if (last != NULL)
-        {
-            while (last->next != NULL)
-                last = last->next;
-            last->next = in;
-        }
-        else
-        {
-            *buf_out = in;
-        }
         *buf_in  = NULL;
         return HB_FILTER_DONE;
     }
