@@ -1,12 +1,16 @@
 /* ports.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
+#ifdef SYS_MINGW
+#define _WIN32_WINNT 0x600
+#endif
+
 #ifdef USE_PTHREAD
 #ifdef SYS_LINUX
 #define _GNU_SOURCE
@@ -38,6 +42,7 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <dlfcn.h>
 #endif
 
 #ifdef SYS_CYGWIN
@@ -50,6 +55,7 @@
 #include <wchar.h>
 #include <mbctype.h>
 #include <locale.h>
+#include <shlobj.h>
 #endif
 
 #ifdef SYS_SunOS
@@ -262,7 +268,14 @@ const char* hb_get_cpu_platform_name()
             return "Intel microarchitecture Silvermont";
         case HB_CPU_PLATFORM_INTEL_HSW:
             return "Intel microarchitecture Haswell";
-
+        case HB_CPU_PLATFORM_INTEL_BDW:
+            return "Intel microarchitecture Broadwell";
+        case HB_CPU_PLATFORM_INTEL_SKL:
+            return "Intel microarchitecture Skylake";
+        case HB_CPU_PLATFORM_INTEL_CHT:
+            return "Intel microarchitecture Airmont";
+        case HB_CPU_PLATFORM_INTEL_KBL:
+            return "Intel microarchitecture Kaby Lake";
         default:
             return NULL;
     }
@@ -327,13 +340,32 @@ static void init_cpu_info()
                     case 0x37:
                     case 0x4A:
                     case 0x4D:
+                    case 0x5A:
+                    case 0x5D:
                         hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_SLM;
                         break;
                     case 0x3C:
+                    case 0x3F:
                     case 0x45:
                     case 0x46:
                         hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_HSW;
                         break;
+                    case 0x3D:
+                    case 0x4F:
+                    case 0x56:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_BDW;
+                        break;
+                    case 0x4C:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_CHT;
+                        break;
+                    case 0x4E:
+                    case 0x5E:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_SKL;
+                        break;
+                    case 0x8E:
+                    case 0x9E:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_KBL;
+                        break;
                     default:
                         break;
                 }
@@ -444,7 +476,7 @@ int hb_platform_init()
 {
     int result = 0;
 
-#if defined(SYS_MINGW) && defined(PTW32_STATIC_LIB)
+#if defined(SYS_MINGW) && defined(PTW32_VERSION)
     result = !pthread_win32_process_attach_np();
     if (result)
     {
@@ -477,6 +509,92 @@ int hb_platform_init()
     return result;
 }
 
+/************************************************************************
+ * Get app data config directory
+ ***********************************************************************/
+void hb_get_user_config_directory( char path[512] )
+{
+    /* Create the base */
+#if defined( SYS_CYGWIN ) || defined( SYS_MINGW )
+#ifndef CSIDL_FLAG_DONT_UNEXPAND
+    /*
+     * XXX: some old MinGW toolchains don't have SHGetKnownFolderPath.
+     *
+     * SHGetFolderPath is deprecated, but this should be no problem in practice.
+     *
+     * Note: explicitly call the Unicode/WCHAR function SHGetFolderPathW.
+     */
+    WCHAR wide_path[MAX_PATH];
+
+    if (SHGetFolderPathW(NULL, CSIDL_APPDATA, NULL, SHGFP_TYPE_CURRENT, wide_path) == S_OK &&
+        WideCharToMultiByte(CP_UTF8, 0, wide_path, -1, path, 512, NULL, NULL) != 0)
+    {
+        path[511] = 0;
+        return;
+    }
+#else
+    WCHAR *wide_path;
+
+    if (SHGetKnownFolderPath(&FOLDERID_RoamingAppData, 0, NULL, &wide_path) == S_OK &&
+        WideCharToMultiByte(CP_UTF8, 0, wide_path, -1, path, 512, NULL, NULL) != 0)
+    {
+        CoTaskMemFree(wide_path);
+        path[511] = 0;
+        return;
+    }
+    else if (wide_path != NULL)
+    {
+        CoTaskMemFree(wide_path);
+    }
+#endif // !defined CSIDL_FLAG_DONT_UNEXPAND
+#elif defined( SYS_LINUX )
+    char *p;
+
+    if ((p = getenv("XDG_CONFIG_HOME")) != NULL)
+    {
+        strncpy(path, p, 511);
+        path[511] = 0;
+        return;
+    }
+    else if ((p = getenv("HOME")) != NULL)
+    {
+        strncpy(path, p, 511);
+        path[511] = 0;
+        int len = strlen(path);
+        strncpy(path + len, "/.config", 511 - len - 1);
+        path[511] = 0;
+        return;
+    }
+#elif defined( __APPLE__ )
+    if (osx_get_user_config_directory(path) == 0)
+    {
+        return;
+    }
+#endif
+
+    hb_error("Failed to lookup user config directory!");
+    path[0] = 0;
+}
+
+/************************************************************************
+ * Get a user config filename for HB
+ ***********************************************************************/
+void hb_get_user_config_filename( char name[1024], char *fmt, ... )
+{
+    va_list args;
+
+    hb_get_user_config_directory( name );
+#if defined( SYS_CYGWIN ) || defined( SYS_MINGW )
+    strcat( name, "\\" );
+#else
+    strcat( name, "/" );
+#endif
+
+    va_start( args, fmt );
+    vsnprintf( &name[strlen(name)], 1024 - strlen(name), fmt, args );
+    va_end( args );
+}
+
 /************************************************************************
  * Get a temporary directory for HB
  ***********************************************************************/
@@ -570,7 +688,7 @@ FILE * hb_fopen(const char *path, const char *mode)
 #endif
 }
 
-HB_DIR* hb_opendir(char *path)
+HB_DIR* hb_opendir(const char *path)
 {
 #ifdef SYS_MINGW
     HB_DIR *dir;
@@ -697,7 +815,11 @@ static uint64_t hb_thread_to_integer( const hb_thread_t* t )
     #if defined( SYS_CYGWIN )
         return (uint64_t)t->thread;
     #elif defined( _WIN32 ) || defined( __MINGW32__ )
+    #if defined(PTW32_VERSION)
         return (uint64_t)(ptrdiff_t)t->thread.p;
+    #else
+        return (uint64_t)t->thread;
+    #endif
     #elif defined( SYS_DARWIN )
         return (unsigned long)t->thread;
     #else
@@ -881,6 +1003,10 @@ void hb_lock_close( hb_lock_t ** _l )
 {
     hb_lock_t * l = *_l;
 
+    if (l == NULL)
+    {
+        return;
+    }
 #if defined( SYS_BEOS )
     delete_sem( l->sem );
 #elif USE_PTHREAD
@@ -960,6 +1086,10 @@ void hb_cond_close( hb_cond_t ** _c )
 {
     hb_cond_t * c = *_c;
 
+    if (c == NULL)
+    {
+        return;
+    }
 #if defined( SYS_BEOS )
 #elif USE_PTHREAD
     pthread_cond_destroy( &c->cond );
@@ -996,6 +1126,11 @@ void hb_clock_gettime( struct timespec *tp )
     tp->tv_nsec = tv.tv_usec * 1000;
 }
 
+void hb_yield(void)
+{
+    sched_yield();
+}
+
 void hb_cond_timedwait( hb_cond_t * c, hb_lock_t * lock, int msec )
 {
 #if defined( SYS_BEOS )
@@ -1128,33 +1263,6 @@ void hb_net_close( hb_net_t ** _n )
     *_n = NULL;
 }
 
-#ifdef SYS_MINGW
-char *strtok_r(char *s, const char *delim, char **save_ptr) 
-{
-    char *token;
-
-    if (s == NULL) s = *save_ptr;
-
-    /* Scan leading delimiters.  */
-    s += strspn(s, delim);
-    if (*s == '\0') return NULL;
-
-    /* Find the end of the token.  */
-    token = s;
-    s = strpbrk(token, delim);
-    if (s == NULL)
-        /* This token finishes the string.  */
-        *save_ptr = strchr(token, '\0');
-    else {
-        /* Terminate the token and make *SAVE_PTR point past it.  */
-        *s = '\0';
-        *save_ptr = s + 1;
-    }
-
-    return token;
-}
-#endif
-
 /************************************************************************
 * OS Sleep Allow / Prevent
 ***********************************************************************/
@@ -1243,7 +1351,7 @@ void hb_system_sleep_private_disable(void *opaque)
         return;
     }
 
-    IOReturn success = IOPMAssertionCreateWithName(kIOPMAssertionTypeNoIdleSleep,
+    IOReturn success = IOPMAssertionCreateWithName(kIOPMAssertPreventUserIdleSystemSleep,
                                                    kIOPMAssertionLevelOn,
                                                    reasonForActivity,
                                                    assertionID);
@@ -1259,3 +1367,34 @@ void hb_system_sleep_private_disable(void *opaque)
     }
 #endif
 }
+
+void * hb_dlopen(const char *name)
+{
+#ifdef SYS_MINGW
+    HMODULE h = LoadLibraryA(name);
+#else
+    void *h = dlopen(name, RTLD_LAZY | RTLD_LOCAL);
+#endif
+
+    return h;
+}
+
+void * hb_dlsym(void *h, const char *name)
+{
+#ifdef SYS_MINGW
+    FARPROC p = GetProcAddress(h, name);
+#else
+    void *p = dlsym(h, name);
+#endif
+    return p;
+}
+
+int hb_dlclose(void *h)
+{
+#ifdef SYS_MINGW
+    return FreeLibrary(h);
+#else
+    return dlclose(h);
+#endif
+}
+
