@@ -1,7 +1,8 @@
 ###############################################################################
 ##
-## This script is coded for minimum version of Python 2.4 .
-## Pyhthon3 is incompatible.
+## This script is coded for minimum version of Python 2.7 .
+##
+## Python3 is incompatible.
 ##
 ## Authors: konablend
 ##
@@ -9,15 +10,18 @@
 
 import fnmatch
 import glob
+import json
 import optparse
 import os
 import platform
+import random
 import re
+import string
 import subprocess
 import sys
 import time
+from datetime import datetime, timedelta
 
-from optparse import OptionGroup
 from optparse import OptionGroup
 from optparse import OptionParser
 from sys import stderr
@@ -80,13 +84,13 @@ class Configure( object ):
     def infof( self, format, *args ):
         line = format % args
         self._log_verbose.append( line )
-        if cfg.verbose >= Configure.OUT_INFO:
+        if self.verbose >= Configure.OUT_INFO:
             self._log_info.append( line )
             stdout.write( line )
     def verbosef( self, format, *args ):
         line = format % args
         self._log_verbose.append( line )
-        if cfg.verbose >= Configure.OUT_VERBOSE:
+        if self.verbose >= Configure.OUT_VERBOSE:
             stdout.write( line )
 
     ## doc is ready to be populated
@@ -95,15 +99,19 @@ class Configure( object ):
         self.build_final  = os.curdir
         self.src_final    = self._final_dir( self.build_dir, self.src_dir )
         self.prefix_final = self._final_dir( self.build_dir, self.prefix_dir )
+        if host.match( '*-*-darwin*' ):
+            self.xcode_prefix_final = self._final_dir( self.build_dir, self.xcode_prefix_dir )
 
-        cfg.infof( 'compute: makevar SRC/    = %s\n', self.src_final )
-        cfg.infof( 'compute: makevar BUILD/  = %s\n', self.build_final )
-        cfg.infof( 'compute: makevar PREFIX/ = %s\n', self.prefix_final )
+        self.infof( 'compute: makevar SRC/    = %s\n', self.src_final )
+        self.infof( 'compute: makevar BUILD/  = %s\n', self.build_final )
+        self.infof( 'compute: makevar PREFIX/ = %s\n', self.prefix_final )
+        if host.match( '*-*-darwin*' ):
+            self.infof( 'compute: makevar XCODE.prefix/ = %s\n', self.xcode_prefix_final )
 
     ## perform chdir and enable log recording
     def chdir( self ):
         if os.path.abspath( self.build_dir ) == os.path.abspath( self.src_dir ):
-            cfg.errln( 'build (scratch) directory must not be the same as top-level source root!' )
+            self.errln( 'build (scratch) directory must not be the same as top-level source root!' )
 
         if self.build_dir != os.curdir:
             if os.path.exists( self.build_dir ):
@@ -126,7 +134,7 @@ class Configure( object ):
         dir = os.path.dirname( args[0] )
         if len(args) > 1 and args[1].find('w') != -1:
             self.mkdirs( dir )
-        m = re.match( '^(.*)\.tmp$', args[0] )
+        m = re.match( '^(.*)\.tmp\..{8}$', args[0] )
         if m:
             self.infof( 'write: %s\n', m.group(1) )
         else:
@@ -135,18 +143,18 @@ class Configure( object ):
         try:
             return open( *args )
         except Exception, x:
-            cfg.errln( 'open failure: %s', x )
+            self.errln( 'open failure: %s', x )
 
     def record_log( self ):
         if not self._record:
             return
         self._record = False
         self.verbose = Configure.OUT_QUIET
-        file = cfg.open( 'log/config.info.txt', 'w' )
+        file = self.open( 'log/config.info.txt', 'w' )
         for line in self._log_info:
             file.write( line )
         file.close()
-        file = cfg.open( 'log/config.verbose.txt', 'w' )
+        file = self.open( 'log/config.verbose.txt', 'w' )
         for line in self._log_verbose:
             file.write( line )
         file.close()
@@ -159,12 +167,12 @@ class Configure( object ):
             if os.access( name, os.X_OK ):
                 return name
             return None
-        
+
         if not os.environ.has_key( 'PATH' ) or os.environ[ 'PATH' ] == '':
             path = os.defpath
         else:
             path = os.environ['PATH']
-        
+
         for dir in path.split( os.pathsep ):
             f = os.path.join( dir, name )
             if os.access( f, os.X_OK ):
@@ -194,6 +202,8 @@ class Configure( object ):
         self.src_dir    = os.path.normpath( options.src )
         self.build_dir  = os.path.normpath( options.build )
         self.prefix_dir = os.path.normpath( options.prefix )
+        if host.match( '*-*-darwin*' ):
+            self.xcode_prefix_dir = os.path.normpath( options.xcode_prefix )
         if options.sysroot != None:
                 self.sysroot_dir = os.path.normpath( options.sysroot )
         else:
@@ -208,6 +218,10 @@ class Configure( object ):
         if os.path.abspath( self.src_dir ) == os.path.abspath( self.build_dir ):
             self.build_dir = os.path.join( self.build_dir, 'build' )
 
+    ## generate a temporary filename - not worried about race conditions
+    def mktmpname( self, filename ):
+        return filename + '.tmp.' + ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(8))
+
 ###############################################################################
 ##
 ## abstract action
@@ -317,7 +331,7 @@ class ShellProbe( Action ):
 ##
 ## returns true if feature successfully compiles
 ##
-##   
+##
 class CCProbe( Action ):
     def __init__( self, pretext, command, test_file ):
         super( CCProbe, self ).__init__( 'probe', pretext )
@@ -358,7 +372,7 @@ class CCProbe( Action ):
 ##
 ## returns true if feature successfully compiles
 ##
-##   
+##
 class LDProbe( Action ):
     def __init__( self, pretext, command, lib, test_file ):
         super( LDProbe, self ).__init__( 'probe', pretext )
@@ -444,7 +458,7 @@ class HostTupleProbe( ShellProbe, list ):
 
         if self.match( '*-*-cygwin*' ):
             self.systemf = self[2][0].upper() + self[2][1:]
-            
+
     ## glob-match against spec
     def match( self, *specs ):
         for spec in specs:
@@ -519,7 +533,7 @@ class IfHost( object ):
 
     def __nonzero__( self ):
         return self.value != None
-        
+
     def __str__( self ):
         return self.value
 
@@ -551,7 +565,7 @@ class ArchAction( Action ):
     def _action( self ):
         self.fail = False
 
-        ## some match on system should be made here; otherwise we signal a warning. 
+        ## some match on system should be made here; otherwise we signal a warning.
         if host.match( '*-*-cygwin*' ):
             pass
         elif host.match( '*-*-darwin11.*' ):
@@ -616,8 +630,8 @@ class CoreProbe( Action ):
         ## clamp
         if self.count < 1:
             self.count = 1
-        elif self.count > 32:
-            self.count = 32
+        elif self.count > 64:
+            self.count = 64
 
         if options.launch:
             if options.launch_jobs == 0:
@@ -673,74 +687,83 @@ class SelectMode( dict ):
 ## Builds are classed into one of the following types:
 ##
 ##  release
-##      must be built from official svn with '/tags/' in the url
+##      must be built from official git at version tag
 ##  developer
-##      must be built from official svn but is not a release
-##  unofficial
-##      all other builds
+##      must be built from official git but is not a release
 ##
 class RepoProbe( ShellProbe ):
     def __init__( self ):
-        svn = 'svn'
-
-        ## Possible the repo was created using an incompatible version than what is
-        ## available in PATH when probe runs. Workaround by checking for file
-        ## .svn/HANDBRAKE_REPO_PROBE which points to a preferred svn executable.
+        # Find script that creates repo info
         try:
-            hrp = os.path.join( cfg.src_dir, '.svn', 'HANDBRAKE_REPO_PROBE' )
-            if os.path.isfile( hrp ) and os.path.getsize( hrp ) > 0:
-                file = cfg.open( hrp, 'r' )
-                line = file.readline().strip()
-                file.close()
-                if line:
-                    svn = line
+            repo_info = os.path.join( cfg.src_dir, 'scripts', 'repo-info.sh' )
+            if not os.path.isfile( repo_info ):
+                cfg.errln( 'Missing required script %s\n', repo_info )
+                sys.exit( 1 )
         except:
-            pass
+            sys.exit( 1 )
 
-        super( RepoProbe, self ).__init__( 'svn info', '%s info %s' % (svn,cfg.src_dir) )
+        super( RepoProbe, self ).__init__( 'repo info', '%s %s' %
+                                            (repo_info, cfg.src_dir) )
 
-        self.url       = 'svn://nowhere.com/project/unknown'
-        self.root      = 'svn://nowhere.com/project'
+        self.url       = 'git://nowhere.com/project/unknown'
+        self.tag       = ''
+        self.tag_hash  = 'deadbeaf'
         self.branch    = 'unknown'
-        self.uuid      = '00000000-0000-0000-0000-000000000000';
+        self.remote    = 'unknown'
         self.rev       = 0
-        self.date      = '0000-00-00 00:00:00 -0000'
+        self.hash      = 'deadbeaf'
+        self.shorthash = 'deadbea'
+        self.date      = datetime(1, 1, 1)
         self.official  = 0
-        self.type      = 'unofficial'
+        self.type      = 'developer'
 
     def _parseSession( self ):
         for line in self.session:
             ## grok fields
-            m = re.match( '([^:]+):\\s+(.+)', line )
+            m = re.match( '([^\=]+)\=(.*)', line )
             if not m:
                 continue
 
             (name,value) = m.groups()
-            if name == 'URL':
+            if name == 'URL' and value != '':
                 self.url = value
-            elif name == 'Repository Root':
-                self.root = value
-            elif name == 'Repository UUID':
-                self.uuid = value
-            elif name == 'Revision':
+            elif name == 'TAG':
+                self.tag = value
+            elif name == 'TAG_HASH':
+                self.tag_hash = value
+            elif name == 'BRANCH':
+                self.branch = value
+            elif name == 'REMOTE':
+                self.remote = value
+            elif name == 'REV':
                 self.rev = int( value )
-            elif name == 'Last Changed Date':
-                # strip chars in parens
-                if value.find( ' (' ):
-                    self.date = value[0:value.find(' (')]
+            elif name == 'DATE':
+                self.date = datetime.strptime(value[0:19], "%Y-%m-%d %H:%M:%S")
+
+                # strptime can't handle UTC offset
+                m = re.match( '^([-+]?[0-9]{2})([0-9]{2})$', value[20:])
+                (hh, mn) = m.groups()
+                utc_off_hour   = int(hh)
+                utc_off_minute = int(mn)
+                if utc_off_hour >= 0:
+                    utc_off = utc_off_hour * 60 + utc_off_minute
                 else:
-                    self.date = value
+                    utc_off = utc_off_hour * 60 - utc_off_minute
+                delta = timedelta(minutes=utc_off)
+                self.date = self.date - delta
 
-        ## grok branch
-        i = self.url.rfind( '/' )
-        if i != -1 and i < len(self.url)-1:
-            self.branch = self.url[i+1:]
+            elif name == 'HASH':
+                self.hash = value
+                self.shorthash = value[:7]
 
-        # type-classification via repository UUID
-        if self.uuid == 'b64f7644-9d1e-0410-96f1-a4d463321fa5':
+        # type-classification via repository URL
+        official_url = 'https://github.com/HandBrake/HandBrake.git' # HTTPS
+        if self.url == 'git@github.com:HandBrake/HandBrake.git':    # SSH
+            self.url = official_url
+
+        if self.url == official_url:
             self.official = 1
-            m = re.match( '([^:]+)://([^/]+)/(.+)', self.url )
-            if m and re.match( '.*tags/.*', m.group( 3 )):
+            if not options.snapshot and self.hash == self.tag_hash:
                 self.type = 'release'
             else:
                 self.type = 'developer'
@@ -748,12 +771,12 @@ class RepoProbe( ShellProbe ):
         self.msg_end = self.url
 
     def _failSession( self ):
-        # Look for svn info in version file.
+        # Look for repo info in version file.
         #
         # Version file would be created manually by source packager.
         # e.g.
-        # $ svn info HandBrake > HandBrake/version.txt
-        # $ tar -czf handbrake-source.tgz --exclude .svn HandBrake
+        # $ HandBrake/scripts/repo-info.sh HandBrake > HandBrake/version.txt
+        # $ tar -czf handbrake-source.tgz --exclude .git HandBrake
         cfg.infof( 'probe: version.txt...' )
         try:
             hvp = os.path.join( cfg.src_dir, 'version.txt' )
@@ -793,8 +816,11 @@ class Project( Action ):
         self.name_upper = self.name.upper()
 
         self.vmajor = 0
-        self.vminor = 10
-        self.vpoint = 2
+        self.vminor = 0
+        self.vpoint = 0
+        self.spoint = 0
+        self.suffix = ''
+        self.special = ''
 
     def _action( self ):
         ## add architecture to URL only for Mac
@@ -803,24 +829,52 @@ class Project( Action ):
         else:
             url_arch = ''
 
-        if repo.type == 'release':
-            self.version = '%d.%d.%d' % (self.vmajor,self.vminor,self.vpoint)
-            url_ctype = ''
-            url_ntype = 'stable'
-            self.build = time.strftime('%Y%m%d') + '00'
-            self.title = '%s %s (%s)' % (self.name,self.version,self.build)
-        elif repo.type == 'developer':
-            self.version = '%dsvn' % (repo.rev)
+        if repo.tag != '':
+            m = re.match( '^([0-9]+)\.([0-9]+)\.([0-9]+)-?(.+)?$', repo.tag )
+            if not m:
+                cfg.errln( 'Invalid repo tag format %s\n', repo.tag )
+                sys.exit( 1 )
+            (vmajor, vminor, vpoint, suffix) = m.groups()
+            self.vmajor = int(vmajor)
+            self.vminor = int(vminor)
+            self.vpoint = int(vpoint)
+            if suffix:
+                self.suffix = suffix
+
+        if repo.type != 'release' or options.snapshot:
+            self.version = repo.date.strftime("%Y%m%d%H%M%S")
+            self.version += '-%s' % (repo.shorthash)
+            if repo.branch != '':
+                self.version += '-%s' % (repo.branch)
+
+            self.debversion = repo.date.strftime("%Y%m%d%H%M%S")
+            self.debversion += '-%s' % (repo.shorthash)
+            if repo.branch != '':
+                self.debversion += '-%s' % (repo.branch)
+
             url_ctype = '_unstable'
             url_ntype = 'unstable'
-            self.build = time.strftime('%Y%m%d') + '01'
-            self.title = '%s svn%d (%s)' % (self.name,repo.rev,self.build)
+            self.build = time.strftime('%Y%m%d', now) + '01'
+            self.title = '%s %s (%s)' % (self.name,self.version,self.build)
         else:
-            self.version = 'rev%d' % (repo.rev)
-            url_ctype = '_unofficial'
-            url_ntype = 'unofficial'
-            self.build = time.strftime('%Y%m%d') + '99'
-            self.title = '%s rev%d (%s)' % (self.name,repo.rev,self.build)
+            m = re.match('^([a-zA-Z]+)\.([0-9]+)$', self.suffix)
+            if not m:
+                # Regular release
+                self.version = '%d.%d.%d' % (self.vmajor,self.vminor,self.vpoint)
+                self.debversion = '%d.%d.%d' % (self.vmajor, self.vminor, self.vpoint)
+                url_ctype = ''
+                url_ntype = 'stable'
+            else:
+                (special, spoint,) = m.groups()
+                self.special = special
+                self.spoint = int(spoint)
+                self.version = '%d.%d.%d-%s.%d' % (self.vmajor,self.vminor,self.vpoint, self.special, self.spoint)
+                self.debversion = '%d.%d.%d~%s.%d' % (self.vmajor, self.vminor, self.vpoint, self.special, self.spoint)
+                url_ctype = '_unstable'
+                url_ntype = 'unstable'
+
+            self.build = time.strftime('%Y%m%d', now) + '00'
+            self.title = '%s %s (%s)' % (self.name,self.version,self.build)
 
         self.url_appcast = 'https://handbrake.fr/appcast%s%s.xml' % (url_ctype,url_arch)
         self.url_appnote = 'https://handbrake.fr/appcast/%s.html' % (url_ntype)
@@ -1111,7 +1165,7 @@ class ConfigDocument:
         else:
             raise ValueError, 'unknown file type: ' + type
 
-        ftmp  = fname + '.tmp'
+        ftmp = cfg.mktmpname(fname)
         try:
             try:
                 file = cfg.open( ftmp, 'w' )
@@ -1133,6 +1187,41 @@ class ConfigDocument:
         except Exception, x:
             cfg.errln( 'failed writing to %s\n%s', fname, x )
 
+###############################################################################
+
+def encodeDistfileConfig():
+    fname = 'distfile.cfg'
+    ftmp = cfg.mktmpname(fname)
+    data = {
+        'disable-fetch':  options.disable_df_fetch,
+        'disable-verify': options.disable_df_verify,
+        'jobs':           options.df_jobs,
+        'verbosity':      options.df_verbosity,
+        'accept-url':     options.df_accept_url,
+        'deny-url':       options.df_deny_url,
+    }
+    try:
+        try:
+            file = cfg.open( ftmp, 'w' )
+            json.dump(data, file)
+            file.write('\n')
+        finally:
+            try:
+                file.close()
+            except:
+                pass
+    except Exception, x:
+        try:
+            os.remove( ftmp )
+        except Exception, x:
+            pass
+        cfg.errln( 'failed writing to %s\n%s', ftmp, x )
+
+    try:
+        os.rename( ftmp, fname )
+    except Exception, x:
+        cfg.errln( 'failed writing to %s\n%s', fname, x )
+
 ###############################################################################
 ##
 ## create cli parser
@@ -1171,9 +1260,21 @@ def createCLI():
 
     ## add hidden options
     cli.add_option( '--xcode-driver', default='bootstrap', action='store', help=optparse.SUPPRESS_HELP )
+
+    ## add general options
     cli.add_option( '--force', default=False, action='store_true', help='overwrite existing build config' )
     cli.add_option( '--verbose', default=False, action='store_true', help='increase verbosity' )
 
+    ## add distfile options
+    grp = OptionGroup( cli, 'Distfile Options' )
+    grp.add_option( '--disable-df-fetch', default=False, action='store_true', help='disable distfile downloads' )
+    grp.add_option( '--disable-df-verify', default=False, action='store_true', help='disable distfile data verification' )
+    grp.add_option( '--df-jobs', default=1, action='store', metavar='N', type='int', help='allow N distfile downloads at once' )
+    grp.add_option( '--df-verbose', default=1, action='count', dest='df_verbosity', help='increase distfile tools verbosity' )
+    grp.add_option( '--df-accept-url', default=[], action='append', metavar='SPEC', help='accept URLs matching regex pattern' )
+    grp.add_option( '--df-deny-url', default=[], action='append', metavar='SPEC', help='deny URLs matching regex pattern' )
+    cli.add_option_group( grp )
+
     ## add install options
     grp = OptionGroup( cli, 'Directory Locations' )
     h = IfHost( 'specify sysroot of SDK for Xcode builds', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
@@ -1204,15 +1305,13 @@ def createCLI():
     h = IfHost( 'enable use of Intel Quick Sync Video hardware acceleration', '*-*-*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-qsv', default=False, action='store_true', help=h )
 
-    h = IfHost( 'enable HWD features', '*-*-*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--enable-hwd', default=False, action='store_true', help=h )
-    
+
     h = IfHost( 'enable use of x265 encoding', '*-*-*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-x265', default=True, action='store_true', help=h )
     grp.add_option( '--disable-x265', dest="enable_x265", action='store_false' )
 
     h = IfHost( 'enable use of fdk-aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--enable-fdk-aac', dest="enable_fdk_aac", default=not host.match( '*-*-darwin*' ), action='store_true', help=h )
+    grp.add_option( '--enable-fdk-aac', dest="enable_fdk_aac", default=False, action='store_true', help=h )
     grp.add_option( '--disable-fdk-aac', dest="enable_fdk_aac", action='store_false' )
 
     h = IfHost( 'enable use of libav aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
@@ -1253,13 +1352,18 @@ def createCLI():
     h = IfHost( 'Build and use local pkg-config', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-local-pkgconfig', default=False, action='store_true', help=h )
 
+    h = IfHost( 'Build extra contribs for flatpak packaging', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--flatpak', default=False, action='store_true', help=h )
     cli.add_option_group( grp )
 
+
     ## add Xcode options
     if host.match( '*-*-darwin*' ):
         grp = OptionGroup( cli, 'Xcode Options' )
         grp.add_option( '--disable-xcode', default=False, action='store_true',
             help='disable Xcode' )
+        grp.add_option( '--xcode-prefix', default=cfg.xcode_prefix_dir, action='store', metavar='DIR',
+            help='specify install dir for Xcode products [%s]' % (cfg.xcode_prefix_dir) )
         grp.add_option( '--xcode-symroot', default='xroot', action='store', metavar='DIR',
             help='specify root of the directory hierarchy that contains product files and intermediate build files' )
         xcconfigMode.cli_add_option( grp, '--xcode-config' )
@@ -1276,6 +1380,13 @@ def createCLI():
     for select in SelectTool.selects:
         select.cli_add_option( grp )
     cli.add_option_group( grp )
+
+    ## add build options
+    grp = OptionGroup( cli, 'Build Options' )
+    grp.add_option( '--snapshot', default=False, action='store_true',
+                    help='Force a snapshot build' )
+    cli.add_option_group( grp )
+
     return cli
 
 ###############################################################################
@@ -1381,6 +1492,8 @@ try:
         if arg == '--verbose':
             verbose = Configure.OUT_VERBOSE
 
+    now = time.gmtime(int(os.environ.get('SOURCE_DATE_EPOCH', time.time())))
+
     ## create main objects; actions/probes run() is delayed.
     ## if any actions must be run earlier (eg: for configure --help purposes)
     ## then run() must be invoked earlier. subequent run() invocations
@@ -1388,7 +1501,9 @@ try:
     cfg   = Configure( verbose )
     host  = HostTupleProbe(); host.run()
 
-    cfg.prefix_dir = ForHost( '/usr/local', ['/Applications','*-*-darwin*'] ).value
+    cfg.prefix_dir = '/usr/local'
+    if host.match( '*-*-darwin*' ):
+        cfg.xcode_prefix_dir = '/Applications'
 
     build = BuildAction()
     arch  = ArchAction(); arch.run()
@@ -1402,7 +1517,6 @@ try:
     class Tools:
         ar    = ToolProbe( 'AR.exe',    'ar' )
         cp    = ToolProbe( 'CP.exe',    'cp' )
-        curl  = ToolProbe( 'CURL.exe',  'curl', abort=False )
         gcc   = ToolProbe( 'GCC.gcc',   'gcc', IfHost( 'gcc-4', '*-*-cygwin*' ))
 
         if host.match( '*-*-darwin*' ):
@@ -1410,14 +1524,13 @@ try:
         else:
             gmake = ToolProbe( 'GMAKE.exe', 'gmake', 'make' )
 
-        m4       = ToolProbe( 'M4.exe',       'm4' )
+        m4       = ToolProbe( 'M4.exe',       'gm4', 'm4' )
         mkdir    = ToolProbe( 'MKDIR.exe',    'mkdir' )
         patch    = ToolProbe( 'PATCH.exe',    'gpatch', 'patch' )
         rm       = ToolProbe( 'RM.exe',       'rm' )
         ranlib   = ToolProbe( 'RANLIB.exe',   'ranlib' )
         strip    = ToolProbe( 'STRIP.exe',    'strip' )
         tar      = ToolProbe( 'TAR.exe',      'gtar', 'tar' )
-        wget     = ToolProbe( 'WGET.exe',     'wget', abort=False )
         yasm     = ToolProbe( 'YASM.exe',     'yasm', abort=False, minversion=[1,2,0] )
         autoconf = ToolProbe( 'AUTOCONF.exe', 'autoconf', abort=False )
         automake = ToolProbe( 'AUTOMAKE.exe', 'automake', abort=False )
@@ -1428,8 +1541,6 @@ try:
         xcodebuild = ToolProbe( 'XCODEBUILD.exe', 'xcodebuild', abort=False )
         lipo       = ToolProbe( 'LIPO.exe',       'lipo', abort=False )
 
-        fetch = SelectTool( 'FETCH.select', 'fetch', ['wget',wget], ['curl',curl] )
-
     ## run tool probes
     for tool in ToolProbe.tools:
         tool.run()
@@ -1541,7 +1652,8 @@ int main ()
   return 0;
 }
 """
-        pthread = LDProbe( 'static pthread', '%s -static' % Tools.gcc.pathname, '-lpthreadGC2', pthread_test )
+        pthreadGC2 = LDProbe( 'static pthread', '%s -static' % Tools.gcc.pathname, '-lpthreadGC2', pthread_test )
+        pthread = LDProbe( 'static pthread', '%s -static' % Tools.gcc.pathname, '-lpthread', pthread_test )
         pthread.run()
 
         bz2_test = """
@@ -1595,7 +1707,7 @@ int main()
     int     rv;
     regex_t exp;
 
-    rv = regcomp(&exp, "^[0-9]+$";", REG_EXTENDED);
+    rv = regcomp(&exp, "^[0-9]+$", REG_EXTENDED);
     if (rv != 0) {
         return 1;
     }
@@ -1616,6 +1728,36 @@ int main()
         regex = LDProbe( 'static regex', '%s -static' % Tools.gcc.pathname, '-lregex', regex_test )
         regex.run()
 
+        strtok_r_test = """
+#include <string.h>
+
+int main ()
+{
+  char *saveptr;
+  strtok_r("String tok string", "tok", &saveptr);
+  return 0;
+}
+"""
+        strtok_r = LDProbe( 'static strtok_r', '%s -static' % Tools.gcc.pathname, '', strtok_r_test )
+        strtok_r.run()
+
+    strerror_r_test = """
+#include <string.h>
+
+int main()
+{
+    /* some implementations fail if buf is less than 80 characters
+       so size it appropriately */
+    char errstr[128];
+    /* some implementations fail if err == 0 */
+    strerror_r(1, errstr, 127);
+    return 0;
+}
+"""
+
+    strerror_r = LDProbe( 'strerror_r', '%s' % Tools.gcc.pathname, '', strerror_r_test )
+    strerror_r.run()
+
     ## cfg hook before doc prep
     cfg.doc_ready()
 
@@ -1628,7 +1770,7 @@ int main()
     args = []
     for arg in Option.conf_args:
         args.append( arg[1] )
-    doc.add( 'CONF.args', ' '.join( args ))
+    doc.add( 'CONF.args', ' '.join(args).replace('$','$$') )
 
     doc.addBlank()
     doc.add( 'HB.title',       project.title )
@@ -1647,19 +1789,23 @@ int main()
     doc.add( 'HB.version.major',  project.vmajor )
     doc.add( 'HB.version.minor',  project.vminor )
     doc.add( 'HB.version.point',  project.vpoint )
+    doc.add( 'HB.version.suffix', project.suffix )
     doc.add( 'HB.version',        project.version )
+    doc.add( 'HB.debversion',     project.debversion )
     doc.add( 'HB.version.hex',    '%04x%02x%02x%08x' % (project.vmajor,project.vminor,project.vpoint,repo.rev) )
 
     doc.add( 'HB.build', project.build )
 
     doc.add( 'HB.repo.url',       repo.url )
-    doc.add( 'HB.repo.root',      repo.root )
-    doc.add( 'HB.repo.branch',    repo.branch )
-    doc.add( 'HB.repo.uuid',      repo.uuid )
+    doc.add( 'HB.repo.tag',       repo.tag )
     doc.add( 'HB.repo.rev',       repo.rev )
-    doc.add( 'HB.repo.date',      repo.date )
-    doc.add( 'HB.repo.official',  repo.official )
+    doc.add( 'HB.repo.hash',      repo.hash )
+    doc.add( 'HB.repo.shorthash', repo.shorthash )
+    doc.add( 'HB.repo.branch',    repo.branch )
+    doc.add( 'HB.repo.remote',    repo.remote )
     doc.add( 'HB.repo.type',      repo.type )
+    doc.add( 'HB.repo.official',  repo.official )
+    doc.add( 'HB.repo.date',      repo.date.strftime("%Y-%m-%d %H:%M:%S") )
 
     doc.addBlank()
     doc.add( 'HOST.spec',    host.spec )
@@ -1690,7 +1836,7 @@ int main()
     else:
         doc.add( 'BUILD.cross.prefix', '' )
 
-    doc.add( 'BUILD.date',   time.strftime('%c') )
+    doc.add( 'BUILD.date',   time.strftime('%c', now) ),
     doc.add( 'BUILD.arch',   arch.mode.mode )
 
     doc.addBlank()
@@ -1700,13 +1846,14 @@ int main()
     doc.add( 'BUILD/',  cfg.build_final + os.sep )
     doc.add( 'PREFIX',  cfg.prefix_final )
     doc.add( 'PREFIX/', cfg.prefix_final + os.sep )
-    
+
     doc.addBlank()
     doc.add( 'FEATURE.local_yasm', int( options.enable_local_yasm ))
     doc.add( 'FEATURE.local_autotools', int( options.enable_local_autotools ))
     doc.add( 'FEATURE.local_cmake', int( options.enable_local_cmake ))
     doc.add( 'FEATURE.local_pkgconfig', int( options.enable_local_pkgconfig ))
     doc.add( 'FEATURE.asm',        'disabled' )
+    doc.add( 'FEATURE.flatpak',    int( options.flatpak ))
     doc.add( 'FEATURE.gtk',        int( not options.disable_gtk ))
     doc.add( 'FEATURE.gtk.update.checks', int( not options.disable_gtk_update_checks ))
     doc.add( 'FEATURE.gtk.mingw',  int( options.enable_gtk_mingw ))
@@ -1714,12 +1861,13 @@ int main()
     doc.add( 'FEATURE.fdk_aac',    int( options.enable_fdk_aac ))
     doc.add( 'FEATURE.libav_aac',  int( options.enable_libav_aac ))
     doc.add( 'FEATURE.qsv',        int( options.enable_qsv ))
-    doc.add( 'FEATURE.hwd',        int( options.enable_hwd ))
     doc.add( 'FEATURE.xcode',      int( not (Tools.xcodebuild.fail or options.disable_xcode or options.cross) ))
     doc.add( 'FEATURE.x265',       int( options.enable_x265 ))
 
     if not Tools.xcodebuild.fail and not options.disable_xcode:
         doc.addBlank()
+        doc.add( 'XCODE.prefix',  cfg.xcode_prefix_final )
+        doc.add( 'XCODE.prefix/', cfg.xcode_prefix_final + os.sep )
         doc.add( 'XCODE.driver', options.xcode_driver )
         if os.path.isabs(options.xcode_symroot):
             doc.add( 'XCODE.symroot', options.xcode_symroot )
@@ -1731,7 +1879,9 @@ int main()
         doc.addBlank()
         if not dlfcn.fail:
             doc.add( 'HAS.dlfcn', 1 )
-        if not pthread.fail:
+        if not pthreadGC2.fail:
+            doc.add( 'HAS.pthreadGC2', 1 )
+        elif not pthread.fail:
             doc.add( 'HAS.pthread', 1 )
         if not bz2.fail:
             doc.add( 'HAS.bz2', 1 )
@@ -1741,6 +1891,12 @@ int main()
             doc.add( 'HAS.iconv', 1 )
         if not regex.fail:
             doc.add( 'HAS.regex', 1 )
+        if strtok_r.fail:
+            doc.add( 'COMPAT.strtok_r', 1 )
+    else:
+        doc.addBlank()
+        if not strerror_r.fail:
+            doc.add( 'HAS.strerror_r', 1 )
 
     doc.addMake( '' )
     doc.addMake( '## define debug mode and optimize before other includes' )
@@ -1760,12 +1916,11 @@ int main()
         select.doc_add( doc )
 
     doc.addBlank()
+    doc.add( 'GCC.archs', arch.mode.mode )
     if build.match( '*-*-darwin*' ):
-        doc.add( 'GCC.archs', arch.mode.mode )
         doc.add( 'GCC.sysroot', cfg.sysroot_dir )
         doc.add( 'GCC.minver', cfg.minver )
     else:
-        doc.add( 'GCC.archs', '' )
         doc.add( 'GCC.sysroot', '' )
         doc.add( 'GCC.minver', '' )
 
@@ -1821,6 +1976,8 @@ int main()
     ## perform
     doc.write( 'make' )
     doc.write( 'm4' )
+    encodeDistfileConfig()
+
     if options.launch:
         Launcher( targets )
 
@@ -1850,7 +2007,7 @@ except AbortError, x:
     try:
         cfg.record_log()
     except:
-        pass        
-    sys.exit( 1 )    
+        pass
+    sys.exit( 1 )
 
 sys.exit( 0 )
